import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for alternative_medicine related titles
alternative_medicine_variants = [
    r"(?i)\bAlternative\sMedicine\sPhysician\b",
    r"(?i)\bAlternative\sHealth\sPhysician\b",
    r"(?i)\bHolistic\sPhysician\b",
    r"(?i)\bNatural\sMedicine\sPhysician\b",
    r"(?i)\bIntegrative\sMedicine\sPhysician\b",
    r"(?i)\bComplementary\sMedicine\sPhysician\b",
    r"(?i)\bAlternative\sMedicine\b",
    r"(?i)\bAlternative Medicine\b",
    r"(?i)\bAlternative Medcine\b",
    r"(?i)\bAlternetive Medicine\b",
    r"(?i)\bAlternative Medecine\b",
    r"(?i)\bAlternativ Medicine\b",
    r"(?i)\bAlternitive Medicine\b",
    r"(?i)\bAltrnative Medicine\b",
    r"(?i)\bAlterantive Medicine\b",
    r"(?i)\bAltrenative Medicine\b",
    r"(?i)\bAlternative Medcin\b",
    r"(?i)\bAlternatuve Medicine\b",
    r"(?i)\bAlternetive Medecine\b",
    r"(?i)\bAlterntive Medicine\b",
    r"(?i)\bHeilpraktiker\b",
    r"(?i)\bHeilpraktikerin\b",
    r"(?i)\bHeilpraktikerin\b",
    r"(?i)\bIntegrative Medicine\b",
    r"(?i)\bIridologist\b",
]

# Exact matches that should be updated
alternative_medicine_exact_matches = {
    'Alternative Medicine Physician',
    'Alternative Medicine Doctor',
    'Holistic Medicine Physician',
    'Integrative Medicine Physician',
    'Natural Medicine Physician',
    'Complementary Medicine Physician',
    'Holistic Health Physician',
    'Alternative Health Physician',
    'Alternative Medicine Specialist',
    'Holistic Medicine Doctor',
    'Integrative Medicine Doctor',
    'Natural Medicine Doctor',
    'Complementary Medicine Doctor',
    'Alternative Medicine Expert',
    'Holistic Health Doctor',
    'Natural Health Physician',
    'Natural Medicine Specialist',
    'Integrative Health Physician',
    'Integrative Medicine Specialist',
    'Alternative Medicine Practitioner',
    'Holistic Medicine Practitioner',
    'Integrative Medicine Practitioner',
    'Natural Medicine Practitioner',
    'Complementary Medicine Practitioner',
    'Alternative Medicine Practitioner Expert',
    'Alternative Medicine Doctor Expert',
    'Holistic Medicine Practitioner Expert',
    'Integrative Medicine Practitioner Expert',
    'Natural Medicine Practitioner Expert',
    'Complementary Medicine Practitioner Expert',
    'Alternative Medicine',

    # Spanish Titles:
    'Médico De Medicina Alternativa',
    'Médico En Medicina Alternativa',
    'Especialista En Medicina Alternativa',

    # Spanish: Alternative Medicine Physician
    'Médico De Salud Holística',
    'Médico En Medicina Holística',
    'Especialista En Medicina Holística',

    # Spanish: Holistic Medicine Physician
    'Médico De Medicina Natural',
    'Médico En Medicina Natural',
    'Especialista En Medicina Natural',

    # Spanish: Natural Medicine Physician
    'Médico De Medicina Integrativa',
    'Médico En Medicina Integrativa',
    'Especialista En Medicina Integrativa',

    # Spanish: Integrative Medicine Physician
    'Médico De Medicina Complementaria',
    'Médico En Medicina Complementaria',
    'Especialista En Medicina Complementaria',

    # Spanish: Complementary Medicine Physician

    # Variations for misspellings and case mistakes for Alternative Medicine Physician and related titles
    'Alternive Medicine Physician',
    'Alternative Medcine Physician',
    'Alternative Medecine Physician',
    'Alternative Medcine Doctor',
    'Alterntive Medicine Physician',
    'Alternitive Medicine Physician',
    'Altrernative Medicine Physician',
    'Alternative Mmedicine Physician',
    'Alternativ Medicine Physician',
    'Alternativ Medcine Physician',
    'Alternative Mdicine Physician',
    'Alterntive Medicine Dr.',
    'Alternative Medcine Dr.',
    'Alternative Medicine Docotr',
    'Altenative Medicine Physician',
    'Alterantive Medicine Physcian',
    'Alternative Medicne Physician',
    'Alternative Meidcine Physician',

    # Case mistakes:
    'Alternative medicine physician',
    'alternative medicine physician',
    'ALTERNATIVE MEDICINE PHYSICIAN',
    'Alternative Medicine physician',
    'alternative Medicine Physician',
    'alternative medicine Physician',
    'alternative Medcine Physician',
    'Alternative Medicine PHYSICIAN',
    'alternative Medicine Physician Expert',

    # Common Misspellings
    'Alternetive Medicine Physician',
    'Alternative Medicin Physician',
    'Alternative Medcine Doctor',
    'Alternitive Medcine Physician',
    'Alternetive Medicine Doc',
    'Alternative Meidicine Physician',
    'Alternative Medicne Physcian',
    'Alternative Medcine Physican',
    'Alterantive Medicine Doctor',
    'Alternative Medicine Pracitioner',
    'Alternative Medicen Physician',
    'Altrernative Medicine Doc',
    'Alternive Medcine Doctor',
    'Alternative Medecine Doctor',
    'Alteernative Medicine Physician',
    'Alternative Medicine',
    'Alternative Medcine',
    'Alternetive Medicine',
    'Alternative Medecine',
    'Alternativ Medicine',
    'Alternitive Medicine',
    'Altrnative Medicine',
    'Alterantive Medicine',
    'Altrenative Medicine',
    'Alternative Medcin',
    'Alternatuve Medicine',
    'Alternetive Medecine',
    'Alterntive Medicine',
    
    # Case-related errors for Alternative Medicine Physician:
    'alternative medicine Physician',
    'Alternative medcine physician',
    'Alternative medicine PHYSICIAN',
    'alternative Medicine Physician',
    'Alternative Medicine PHYsician',
    'alternative Medicine PHYsician',
    'ALTERNATIVE MEDICINE physician',
    'Alternative Medicne Physician',
    'alternative Medicine Physican',
    'Alternative medicine Phycian',
    'altnernative medicine physician',
    'Alternative medicne PHYSICIAN',
    'alternative medicine',
    'ALTERNATIVE MEDICINE',
    'Alternative MEDICINE',
    'aLTERNATIVE mEDICINE',
    'aLTERnAtIVE medicine',
    'alTeRnAtIvE mEdIcInE',

    # Spanish Variants for Alternative Medicine Physician titles:
    'Médico De Medicina Alternativa',
    'Médico En Medicina Alternativa',
    'Especialista En Medicina Alternativa',
    'Médico De Medicina Natural',
    'Médico En Medicina Natural',
    'Especialista En Medicina Natural',
    'Médico De Medicina Integrativa',
    'Médico En Medicina Integrativa',
    'Especialista En Medicina Integrativa',
    'Médico De Medicina Complementaria',
    'Médico En Medicina Complementaria',
    'Especialista En Medicina Complementaria',
    'Médico Holístico',
    'Médico De Salud Holística',
    'Médico De Medicina Complementaria',
    'Médico De Medicina Preventiva',
    'Médico Integrativo',
    'Especialista En Medicina Holística',
    'Medicina Alternativa',
    'Médico de Medicina Alternativa',
    'Médico Alternativo',
    'Especialista en Medicina Alternativa',
    'Médico en Medicina Alternativa',
    'Medicina Natural',
    'Especialista en Medicina Natural',
    'Médico Natural',
    'Salud Alternativa',
    'Tratamientos Alternativos',
    'Medicina Complementaria',
    'Medicina Holística',
    'Medicina Integrativa',

    # Doctor Forms:
    'Alternative Medicine Doctor',
    'Holistic Medicine Doctor',
    'Integrative Medicine Doctor',
    'Natural Medicine Doctor',
    'Complementary Medicine Doctor',
    'Holistic Health Doctor',
    'Alternative Health Doctor',
    'Doctor of Integrative Medicine',
    'Doctor of Holistic Medicine',
    'Doctor of Alternative Medicine',
    'Doctor of Natural Medicine',
    'Doctor of Complementary Medicine',

    # Specialist Forms:
    'Alternative Medicine Specialist',
    'Holistic Medicine Specialist',
    'Natural Medicine Specialist',
    'Integrative Medicine Specialist',
    'Complementary Medicine Specialist',
    'Alternative Medicine Expert',
    'Holistic Medicine Expert',
    'Integrative Medicine Expert',
    'Natural Medicine Expert',
    'Complementary Medicine Expert',
    'Alternative Medicine Consultant',
    'Holistic Medicine Consultant',
    'Integrative Medicine Consultant',
    'Natural Medicine Consultant',
    'Complementary Medicine Consultant',
    'Homeopathic Physician',
    'Functional Medicine',
    'Chiropractic Physician',
    'Functional',
}

# Define patterns for  & Resident & Professor (these should NOT be changed)
alternative_medicine_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Alternative Medicine Physician
mask_alternative_medicine = df['speciality'].str.contains('|'.join(alternative_medicine_variants), case = False,
                                                         na = False, regex = True) | \
                           df['speciality'].isin(alternative_medicine_exact_matches)
mask_alternative_medicine_exclusions = df['speciality'].isin(alternative_medicine_exclusions)

# Final mask: Select Alternative Medicine Physician
mask_alternative_medicine_final = mask_alternative_medicine & ~mask_alternative_medicine_exclusions

# Store the original values that will be replaced
original_alternative_medicine_values = df.loc[mask_alternative_medicine_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_alternative_medicine_final, 'speciality'] = 'Alternative Medicine Physician'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Alternative Medicine Physician", 'green'))
print(df.loc[mask_alternative_medicine_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_alternative_medicine_values = df.loc[mask_alternative_medicine_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Alternative Medicine Physician", "cyan"))
for original_alternative_medicine_value in original_alternative_medicine_values:
    print(f"✅ {original_alternative_medicine_value} → Alternative Medicine Physician")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Alternative Medicine Physician:", 'red'))
print(grouped_alternative_medicine_values)

# Print summary
matched_count_alternative_medicine = mask_alternative_medicine_final.sum()

# Print results
print(
    colored(f"\nTotal values matched and changed (Stage 1) to Alternative Medicine Physician: {matched_count_alternative_medicine}",
            'red'))


from setuptools import setup, find_packages

setup(
    name='JobTitleTransformer',  # Must be unique on PyPI
    version='0.1.16',
    packages=find_packages(), # include=["jobtitletransformer"]
    package_data={"JobTitleTransformer": ["job_scripts/*.py"]},
    install_requires=[
        'pandas',
        'numpy',
        'nltk',
        'termcolor',
        'tabulate',
        "scikit-learn",
        "sentence-transformers",
        "faiss-cpu",
        "torch"
    ],
    author='Prasanna UTHAMARAJ',
    author_email='prasanna.uthamaraj@informa.com',  # Optional but nice
    description='A healthcare-focused job title transformation pipeline.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/PrasannaDataBus/JobTitleTransformer',
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License'
    ],
    python_requires='>=3.7',
    entry_points={
        'console_scripts': [
            'jobtitle-transformer = JobTitleTransformer.runner:run_pipeline',
        ],
    }
)


import os
from termcolor import colored
import pkg_resources
import pandas as pd


def run_pipeline(df):
    # List of scripts in order of execution
    ordered_scripts = [
        "_encode_decode.py",
        "Acupuncturist.py",
        "ARNP.py",
        "AestheticAntiAgingPhy.py",
        "AestheticPlasticSurgeon.py",
        "AestheticPhysician.py",
        "AlternativeMedicinePhysician.py",
        "Anaesthesist.py",
        "Anatomist.py",
        "Andrologist.py",
        "AngiologistPhlebologist.py",
        "AntiagingPhysician.py",
        "BiologistBiochemist.py",
        "BiomedicalPhysician.py",
        "Cardiologist.py",
        "ClinicManager.py",
        "ClinicalStaff.py",
        "CompanyCEO.py",
        "CompanyManager.py",
        "CompanyMedicalStaff.py",
        "CompanyRepresentative.py",
        "Consultant.py",
        "CosmeticDermCosmeticSurgeon.py",
        "CosmetologistMD.py",
        "Cytologist.py",
        "DentalAssistant.py",
        "DentalSurgeon.py",
        "Dentist.py",
        "DermatologicSurgeon.py",
        "Dermatologist.py",
        "Distributor.py",
        "Employee.py",
        "Endocrinologist.py",
        "ENTPhysician.py",
        "Esthetician.py",
        "EventManager.py",
        "FacialPlasticSurgeon.py",
        "GeneralPhysician.py",
        "GeneralSurgeon.py",
        "Genetician.py",
        "Geriatrist.py",
        "GynecologistObstetrician.py",
        "HairTransplantSurgeon.py",
        "Headofdepartment.py",
        "Hematologist.py",
        "HospitalManager.py",
        "Immunologist.py",
        "Influencer.py",
        "InternalMedicine.py",
        "LaboratoryMedicine.py",
        "Lawyer.py",
        "LegalCounsel.py",
        "LPN.py",
        "MarketingManager.py",
        "MaxillofacialSurgeon.py",
        "MedicalAdvisor.py",
        "MedicalSpaManager.py",
        "NeonatologistNeurologist.py",
        "Neurophysiologist.py",
        "NonmedicalStaff.py",
        "NursePractitioner.py",
        "Nutritionist.py",
        "OculoplasticSurgeon.py",
        "Oncologist.py",
        "Ophthalmologist.py",
        "OralMaxillofacialSurgeon.py",
        "Orthodontist.py",
        "Orthopedist.py",
        "Owner.py",
        "Pathologist.py",
        "Pediatrician.py",
        "Pharmacist.py",
        "Pharmacologist.py",
        "Photographer.py",
        "PhysicianAssistant.py",
        "Physiotherapist.py",
        "PlasticReconstructiveSurgeon.py",
        "PlasticSurgeon.py",
        "PracticeManager.py",
        "Press.py",
        "ProductProjectManager.py",
        "ProfessorTeacher.py",
        "Psychiatrist.py",
        "Psychologist.py",
        "Psychotherapist.py",
        "Radiologist.py",
        "RN.py",
        "ResearchScientist.py",
        "Rheumatologist.py",
        "Sales.py",
        "SportsMedicinePhysician.py",
        "Stomatologist.py",
        "Student.py",
        "Trichologist.py",
        "Urologist.py",
        "VascularSurgeon.py",
        "TransformationHub.py"
    ]

    # Use a shared globals dict and inject df
    globals_dict = {"df": df}

    # Execute each script
    for script_name in ordered_scripts:
        # Get the path to the script from the installed package
        script_path = pkg_resources.resource_filename('JobTitleTransformer', f'job_scripts/{script_name}')
        print(f"\nExecuting: {script_name}")

        # Open the script and execute it
        with open(script_path, "rb") as f:
            code = compile(f.read(), script_path, 'exec')
            exec(code, globals_dict)

        # Update df after each script (optional but safe)
        df = globals_dict["df"]

    return df


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Head of department related titles

head_of_department_variants = [
    r"(?i)\bHead\s?of\s?Department\b",  # Standard title

    # Common misspellings and case errors
    r"(?i)\bHead\s?of\s?Depatment\b",
    r"(?i)\bHead\s?of\s?Deparment\b",
    r"(?i)\bHead\s?of\s?Departement\b",
    r"(?i)\bHOD\b",
    r"(?i)\bHead Of Service\b",
    r"(?i)\bClinical Director\b",
    r"(?i)\bChief Medical Officer\b",
    r"(?i)\bMedical Cio\b",
    r"(?i)\bHead Of Inject\b",
    r"(?i)\bHead Of Surgical Operation\b",
    r"(?i)\bHead Of Skin Solutions\b",
    r"(?i)\bHead Of Clinical Facilitation & Digital Deployment\b",

    # Spanish variants
    r"(?i)\bJefe\s?de\s?Departamento\b",
    r"(?i)\bDirector\s?de\s?Departamento\b",

    # Other possible variations
    r"(?i)\bDepartment\s?Head\b",
    r"(?i)\bChief\s?of\s?Department\b",
    r"(?i)\bHead\s?of\s?Division\b",
    r"(?i)\bDirector\s?of\s?Department\b",
    r"(?i)\bHead Of Qc Qp\b",
    r"(?i)\bHead Of Department Of Plastic Surgery\b",
    r"(?i)\bHead Of The Department Of Facial & Plastic Surgery\b",
    r"(?i)\bHead Of Department\b",
    r"(?i)\bHead Of Service\b",
    r"(?i)\bHead Of Surgical Operation\b",
    r"(?i)\bHead Of Qc Qp\b",
    r"(?i)\bDirctor\b",
    r"(?i)\bHead Of Service Department\b",
    r"(?i)\bHead Of Skin Solutions\b",
    r"(?i)\bDiretor\b",
    r"(?i)\bClinical Director\b",
    r"(?i)\bHead Of Department Of Plastic Surgery\b",
    r"(?i)\bSupervisor Molbio\b",
    r"(?i)\bField Services Supervisor\b",
    r"(?i)\bNurse Supervisor\b",
    r"(?i)\bSenior Clinical Director\b",
    r"(?i)\bHead Of Clinical Facilitation & Digital Deployment\b",
    r"(?i)\bGrassroots Supervisor\b",
    r"(?i)\bHead Of The Department Of Facial & Plastic Surgery\b",
    r"(?i)\bClinical Director Of Nursing Neuroscience Unit\b",
    r"(?i)\bHead Of Inject\b",
    r"(?i)\bField Supervisor\b",
    r"(?i)\bDivisional Clinical Director\b",
    r"(?i)\bManagement Supervisor\b",
    r"(?i)\bAnatomic Pathology Supervisor\b",
    r"(?i)\bAesthetics Clinical Director\b",
    r"(?i)\bDirector Of Department\b",
    r"(?i)\bHead Of Surgery Department\b",
    r"(?i)\bDir Of Administration\b",
]

# Exact matches that should be updated
head_of_department_exact_matches = {
    "Head of Department",
    "Head of Depatment",
    "Head of Deparment",
    "Head of Departement",
    "HOD",
    # Spanish variants
    "Jefe de Departamento",
    "Director de Departamento",
    # Other possible variations
    "Department Head",
    "Chief of Department",
    "Head of Division",
    "Director of Department",
    'Director',
    "Diretor",
    "Dirctor",
    "Directer",
    "Directar",
    "Directr",
    "Drector",
    "Dierctor",
    "Dorector",
    "Driector",
    'Supervisor',
    'supervisor',
    'Ami Director',
    'Directornuclear Medicine',
    'Technical Director',
    'Director Of Information Department',
    'Directors PaAdministrator',
    'Director Medico',
    'Clinical Transformation Director',
    'Head Of Im & T',
    'Chief Clinical Officer Digital Health',
    'Director Of Labor & Delivery',
    'Head Of Beauty Department At Largest Pharm Retailer In Caucasus',
    'DirectorProvider',
    'Director of Training',
    'Director of non-surgical serives',
    'Cosmetic Director',
    'Aesthetic Director',
    'Dir',
    'Medcal Director',
    'Director Of Physician Services',
    'Administrator Of Physician Services',
    'Assistant Deputy Minister Physician & Provider Services',
    'Assistant Physician Director Primary Care',
    'Clincial Physician Executive',
    'Physician Executive',
    'Physician Executive For Integrated Regina Health',
    'Physician Executive Integrated Northern Health',
    'Regional Physician Executive',
    'Supervisor Dermatology Services Puyallup',
    'Supervisor Theatrical Cosmetology',
    'Supervisor V4650 & Practice Physician',
    'System Physician Executive',
    'Chief Of Clinical Development',
    'DOS',
    'Director of surgical services',
}

# Define patterns (these should NOT be changed)
head_of_department_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Oral Maxillofacial Surgeon Clinical Director',
}

# Create a mask for Head of department
mask_head_of_department = df['speciality'].str.contains('|'.join(head_of_department_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(head_of_department_exact_matches)
mask_head_of_department_exclusions = df['speciality'].isin(head_of_department_exclusions)

# Final mask: Select Head of department
mask_head_of_department_final = mask_head_of_department & ~mask_head_of_department_exclusions

# Store the original values that will be replaced
original_head_of_department_values = df.loc[mask_head_of_department_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_head_of_department_final, 'speciality'] = 'Head of department'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Head of department", 'green'))
print(df.loc[mask_head_of_department_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_head_of_department_values = df.loc[mask_head_of_department_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Head of department", "cyan"))
for original_head_of_department_value in original_head_of_department_values:
    print(f"✅ {original_head_of_department_value} → Head of department")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Head of department:", 'red'))
print(grouped_head_of_department_values)

# Print summary
matched_count_head_of_department = mask_head_of_department_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Head of department: "
        f"{matched_count_head_of_department}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Distributor related titles

distributor_variants = [
    r"(?i)\bdistributor\b",
    r"(?i)\bdistributors\b",
    r"(?i)\bwholesaler\b",
    r"(?i)\bwholesale\s?distributor\b",
    r"(?i)\bsupplier\b",
    r"(?i)\bproduct\s?supplier\b",
    r"(?i)\bvendor\b",
    r"(?i)\bsales\s?representative\b",
    r"(?i)\bdistribuidor\b",
    r"(?i)\bdistribuidores\b",
    r"(?i)\bmayorista\b",
    r"(?i)\bsuministrador\b",
    r"(?i)\bvendedor\b",
    r"(?i)\brepresentante\s?de\s?ventas\b",
    # Misspellings
    r"(?i)\bdistrbutor\b",
    r"(?i)\bdisttributor\b",
    r"(?i)\bdistrbutors\b",
    r"(?i)\bdistbtr\b",
    r"(?i)\bdistrubutor\b",
    # Other Possible Variations
    r"(?i)\bdistribution\s?manager\b",
    r"(?i)\bdistribution\s?coordinator\b",
    r"(?i)\bsales\s?distributor\b",
    r"(?i)\bsupply\s?chain\s?distributor\b",
    r"(?i)\bdelivery\s?distributor\b",
    r"(?i)\bdealer\b",
    r"(?i)\bManufacturing\b",
    r"(?i)\bDistribucion Nuticosmetica\b",
    r"(?i)\bFrangrance & Cosmetic\b",
    r"(?i)\bFrangrance & Cosmetic\b",
    r"(?i)\bComerciante\b",
    r"(?i)\bDistributer\b",
    r"(?i)\bSourcing Agent\b",
    r"(?i)\bIndependent Rep\b",
    r"(?i)\bBiomaterials\b",
    r"(?i)\bComprador\b",
    r"(?i)\bDistributeur\b",
    r"(?i)\bDistributorHolistic Skin Expert\b",
    r"(?i)\bDistruteur\b",
    r"(?i)\bBiomedical Equipment\b",
]

# Exact matches that should be updated
distributor_exact_matches = {
    "Distributor",
    "Distributors",
    "Wholesaler",
    "Wholesale Distributor",
    "Supplier",
    "Product Supplier",
    "Vendor",
    "Sales Representative",
    "Distribuidor",
    "Distribuidores",
    "Mayorista",
    "Sumistrador",
    "Vendedor",
    "Representante de Ventas",
    "Distrbutor",
    "Disttributor",
    "Distbtr",
    "Distrubutor",
    "Distribution Manager",
    "Distribution Coordinator",
    "Sales Distributor",
    "Supply Chain Distributor",
    "Delivery Distributor",
    "Dealer",
    'Orthopedics Instruments',
    'Wholesale Trader Of Gluathione & Dermal Fillers',
    'Wholesale Accounts Director',
    'Equipment',
    'Aesthetic seller',
}

# Define patterns (these should NOT be changed)
distributor_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Distributor
mask_distributor = df['speciality'].str.contains('|'.join(distributor_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(distributor_exact_matches)
mask_distributor_exclusions = df['speciality'].isin(distributor_exclusions)

# Final mask: Select Distributor
mask_distributor_final = mask_distributor & ~mask_distributor_exclusions

# Store the original values that will be replaced
original_distributor_values = df.loc[mask_distributor_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_distributor_final, 'speciality'] = 'Distributor'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Distributor", 'green'))
print(df.loc[mask_distributor_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_distributor_values = df.loc[mask_distributor_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Distributor", "cyan"))
for original_distributor_value in original_distributor_values:
    print(f"✅ {original_distributor_value} → Distributor")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Distributor:", 'red'))
print(grouped_distributor_values)

# Print summary
matched_count_distributor = mask_distributor_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Distributor: "
        f"{matched_count_distributor}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Geriatrist related titles

geriatrist_variants = [
    r"(?i)\bGeriatrist\b",

    # Common misspellings and case errors
    r"(?i)\bGeritrist\b",
    r"(?i)\bGeriatrist\b",
    r"(?i)\bGeriatirist\b",
    r"(?i)\bGeriatrician\b",

    # Spanish variants
    r"(?i)\bGeriatra\b",
    r"(?i)\bEspecialista\s?en\s?Geriatría\b",
    r"(?i)\bMédico\s?Geriatra\b",

    # Other possible variations
    r"(?i)\bDoctor\s?Geriatrist\b",
    r"(?i)\bDoctor\s?en\s?Geriatría\b",
    r"(?i)\bGeriatric\s?Specialist\b",
    r"(?i)\bGeriatric\s?Physician\b",
    r"(?i)\bLicensed\s?Geriatrist\b",
    r"(?i)Geriatrics",
]

# Exact matches that should be updated
geriatrist_exact_matches = {
    "Geriatrist",
    "Geritrist",
    "Geriatirist",
    "Geriatrician",
    "Geriatra",
    "Especialista en Geriatría",
    "Médico Geriatra",
    # Other possible variations
    "Doctor Geriatrist",
    "Doctor en Geriatría",
    "Geriatric Specialist",
    "Geriatric Physician",
    "Licensed Geriatrist",
    'Geriatry Antiaging Laser',
}

# Define patterns (these should NOT be changed)
geriatrist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Geriatrist
mask_geriatrist = df['speciality'].str.contains('|'.join(geriatrist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(geriatrist_exact_matches)
mask_geriatrist_exclusions = df['speciality'].isin(geriatrist_exclusions)

# Final mask: Select Geriatrist
mask_geriatrist_final = mask_geriatrist & ~mask_geriatrist_exclusions

# Store the original values that will be replaced
original_geriatrist_values = df.loc[mask_geriatrist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_geriatrist_final, 'speciality'] = 'Geriatrist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Geriatrist", 'green'))
print(df.loc[mask_geriatrist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_geriatrist_values = df.loc[mask_geriatrist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Geriatrist", "cyan"))
for original_geriatrist_value in original_geriatrist_values:
    print(f"✅ {original_geriatrist_value} → Geriatrist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Geriatrist:", 'red'))
print(grouped_geriatrist_values)

# Print summary
matched_count_geriatrist = mask_geriatrist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Geriatrist: "
        f"{matched_count_geriatrist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Pathologist related titles

pathologist_variants = [
    # Standard Titles & Variants
    r"(?i)\bPathologist\b",
    r"(?i)\bClinical Pathologist\b",
    r"(?i)\bMedical Pathologist\b",
    r"(?i)\bAnatomic Pathologist\b",
    r"(?i)\bForensic Pathologist\b",
    r"(?i)\bPathology Consultant\b",
    r"(?i)\bConsultant Pathologist\b",
    r"(?i)\bLaboratory Pathologist\b",
    r"(?i)\bHistopathologist\b",
    r"(?i)\bHematopathologist\b",
    r"(?i)\bMolecular Pathologist\b",
    r"(?i)\bGeneral Pathologist\b",
    r"(?i)\bPediatric Pathologist\b",
    r"(?i)\bPathology\b",

    # Misspellings & Typographical Errors
    r"(?i)\bPathologiest\b",
    r"(?i)\bPatholigist\b",
    r"(?i)\bPatologist\b",
    r"(?i)\bPatholigist\b",
    r"(?i)\bPthologist\b",
    r"(?i)\bPatholigist\b",

    # Case Variations
    r"(?i)\bpathologist\b",
    r"(?i)\bPaThOlOgIsT\b",
    r"(?i)\bPATHOLOGIST\b",
    r"(?i)\bPaThOlOgIsT\b",

    # Spanish Variants
    r"(?i)\bPatólogo\b",
    r"(?i)\bPatóloga\b",
    r"(?i)\bPatólogo\s?Clínico\b",
    r"(?i)\bPatólogo\s?Médico\b",
    r"(?i)\bPatólogo\s?Anatómico\b",
    r"(?i)\bPatólogo\s?Forense\b",
    r"(?i)\bConsultor\s?Patólogo\b",
    r"(?i)\bPatólogo\s?Consultor\b",
    r"(?i)\bLaboratorio\s?Patólogo\b",
    r"(?i)\bHematopatólogo\b",
    r"(?i)\bPatólogo\s?Molecular\b",
    r"(?i)\bPatólogo\s?General\b",
    r"(?i)\bPatólogo\s?Pediátrico\b",

    # Other Possible Variations
    r"(?i)\bPathology\s?Consultant\b",
    r"(?i)\bLaboratory\s?Pathologist\b",
    r"(?i)\bPathologist\s?Specialist\b",
    r"(?i)\bCertified\s?Pathologist\b",
    r"(?i)\bSenior\s?Pathologist\b",
    r"(?i)\bJunior\s?Pathologist\b",
    r"(?i)\bExpert\s?Pathologist\b",
    r"(?i)\bMedical\s?Pathology\s?Consultant\b",
]

# Exact matches that should be updated
pathologist_exact_matches = {
    "Pathologist",
    "Clinical Pathologist",
    "Medical Pathologist",
    "Anatomic Pathologist",
    "Forensic Pathologist",
    "Pathology Consultant",
    "Consultant Pathologist",
    "Laboratory Pathologist",
    "Histopathologist",
    "Hematopathologist",
    "Molecular Pathologist",
    "General Pathologist",
    "Pediatric Pathologist",
    "Pathologiest",
    "Patholigist",
    "Patologist",
    "Pthologist",
    "Patólogo",
    "Patóloga",
    "Patólogo Clínico",
    "Patólogo Médico",
    "Patólogo Anatómico",
    "Patólogo Forense",
    "Consultor Patólogo",
    "Patólogo Consultor",
    "Laboratorio Patólogo",
    "Hematopatólogo",
    "Patólogo Molecular",
    "Patólogo General",
    "Patólogo Pediátrico",
    "Pathologist Specialist",
    "Certified Pathologist",
    "Senior Pathologist",
    "Junior Pathologist",
    "Expert Pathologist",
    "Medical Pathology Consultant",
    'Biopathologist Antiaging Medicine',
    'Medica Patologista',
}

# # Define patterns (these should NOT be changed)
# pathologist_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

pathologist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Client Developer - Pathology & Dermatology',
}

# Create a mask for Pathologist
mask_pathologist = df['speciality'].str.contains('|'.join(pathologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(pathologist_exact_matches)

# mask_pathologist_exclusions = df['speciality'].str.contains(pathologist_exclusions, case=False, na=False, regex=True)
mask_pathologist_exclusions = df['speciality'].isin(pathologist_exclusions)

# Final mask: Select Pathologist
mask_pathologist_final = mask_pathologist & ~mask_pathologist_exclusions

# Store the original values that will be replaced
original_pathologist_values = df.loc[mask_pathologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_pathologist_final, 'speciality'] = 'Pathologist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Pathologist", 'green'))
print(df.loc[mask_pathologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_pathologist_values = df.loc[mask_pathologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Pathologist", "cyan"))
for original_pathologist_value in original_pathologist_values:
    print(f"✅ {original_pathologist_value} → Pathologist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Pathologist:", 'red'))
print(grouped_pathologist_values)

# Print summary
matched_count_pathologist = mask_pathologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Pathologist: "
        f"{matched_count_pathologist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Clinical Staff related titles

clinical_staff_variants = [
    r"(?i)\bClinical\s?Staff\b",
    r"(?i)\bClinical\s?Team\b",
    r"(?i)\bClinical\s?Support\s?Staff\b",
    r"(?i)\bHealth\s?Care\s?Team\b",
    r"(?i)\bHealthcare\s?Provider\s?Staff\b",
    r"(?i)\bNursing\s?Staff\b",
    r"(?i)\bSupport\s?Staff\b",
    r"(?i)\bClinical\s?Healthcare\s?Staff\b",
    r"(?i)\bAnalistaassistente\b",
    r"(?i)\bAesthetic Facilitator\b",
    r"(?i)\bAesthetic Mediclinic\b",
    r"(?i)\bAesthetic Clinician\b",
    r"(?i)\bAesthetic Device\b",
    r"(?i)\bSeniorclinicalqualitycoordinator\b",
    r"(?i)\bClinical Resource Nurse\b",
    r"(?i)\bDirectrice Centre Laser\b",
    r"(?i)\bAesthetic & Laser\b",
    r"(?i)\bClinical Software Nurse\b",
    r"(?i)\bPhlebotomy\b",
    r"(?i)\bChief Clinical Offier\b",
    r"(?i)\bChief Clinical Information Officer\b",
    r"(?i)\bClinical Operation\b",
    r"(?i)\bCoordinador De Quirofano\b",
    r"(?i)\bPlastic Surgery Trauma Co-Ordinator\b",
    r"(?i)\bClinical Resource Nurse\b",
    r"(?i)\bLaser & Light Therapy Industry\b",
    r"(?i)\bMlso\b",
    r"(?i)\bSeniorclinicalqualitycoordinator\b",
    r"(?i)\bDirectrice Centre Laser\b",
    r"(?i)\bLab Technician\b",
    r"(?i)\bAesthetic & Laser\b",
    r"(?i)\bPhlebotomy\b",
    r"(?i)\bClinical Software Nurse\b",
    r"(?i)\bAesthetic Mediclinic\b",
    r"(?i)\bChief Clinical Offier\b",
    r"(?i)\bChief Clinical Information Officer\b",
    r"(?i)\bClinical Staff\b",
    r"(?i)\bLab Leader\b",
    r"(?i)\bClinical Operation\b",
    r"(?i)\bCoordinador De Quirofano\b",
    r"(?i)\bAnalistaassistente\b",
    r"(?i)\bAesthetic Clinician\b",
    r"(?i)\bMedical Lab Technologist\b",
    r"(?i)\bMedical Lab\b",
    r"(?i)\bLab Director\b",
    r"(?i)\bHealthcare Staff\b",
    r"(?i)\bLab Specialist\b",
    r"(?i)\bPlastic Surgery Trauma Co-Ordinator\b",
    r"(?i)\bMedical Lab Teachnaligi\b",
    r"(?i)\bLab Assistant In Government Hospital\b",
    r"(?i)\bAesthetic Facilitator\b",
    r"(?i)\bAesthetic Device\b",
    r"(?i)\bClinical Education\b",
    r"(?i)\bClinical Specialist\b",
    r"(?i)\bClincial Specialist\b",
    r"(?i)\bSenior Support Specialist\b",
    r"(?i)\bHealthcare Professional\b",
    r"(?i)\bHealthcare\s?Staff\b",
    r"(?i)\bLead Laser Technician\b",
    r"(?i)\bLaser Technician Lead\b",
    r"(?i)\bLazer Technician\b",
    r"(?i)\bAnesthesiologist Technician\b",
    r"(?i)\bX-Ray Technician\b",
    r"(?i)\bMedical Laboratory Technician\b",
    r"(?i)\bPharmacy Technician\b",
    r"(?i)\bMental Health Technician\b",
    r"(?i)\bCae  & Laser Technician\b",
    r"(?i)\bEndoscopy Technician\b",
    r"(?i)\bBiomedical Technician\b",
    r"(?i)\bHnd Pharmacy Technician\b",
    r"(?i)\bEpid  Insp  Technician\b",
]

# Exact matches that should be updated
clinical_staff_exact_matches = {
    'Clinical Staff',
    'Clinical Team',
    'Clinical Support Staff',
    'Health Care Team',
    'Healthcare Provider Staff',
    'Nursing Staff',
    'Support Staff',
    'Clinical Healthcare Staff',
    'Lab',
    'lab',
    'Lead',
    'lead',
    'Laser',
    'laser',

    # Common misspellings for Clinical Staff titles
    'Clincal Staff',
    'Clinicial Staff',
    'Clincial Team',
    'Clinical Teamm',
    'Clinical Support Staffs',

    # Case-related errors for Clinical Staff titles
    'CLINICAL STAFF',
    'clinical STAFF',
    'clinical staff',
    'CLINICAL Team',
    'Healthcare Team',
    'support STAFF',

    # Variants of Clinical Staff for different types in Spanish
    'Personal Clínico',
    'Personal Médico',
    'Equipo Clínico',
    'Equipo de Atención Médica',
    'Personal de Apoyo Clínico',
    'Personal de Salud',
    'Personal de Soporte',
    'Equipo de Proveedores de Atención Médica',
    'Personal de Enfermería',
    'Personal Clínico de Salud',

    # Other possible variations (Including Doctor forms, Specialist forms)
    'Medical Support Staff',
    'Clinical Care Staff',
    'Clinical Providers',
    'Medical Support Team',

    # other values from data sources
    'Mlso',
    'Lead',
    'Laser & Light Therapy Industry',
    'Laser',
    'laser',
    'Cco',
    'Ami Coordinator',
    'Director Of Operating Room',
    'Clinical Nurse Coordinator',
    'Surgical Technologist',
    'Coordinator Of New Life Plastic & Dermatology Plovdiv',
    'Healthcare Staff',
    'Healthcar Staff',
    'Healtcare Staff',
    'HealthCare Staff',
    'HEALTHCARE STAFF',
    'healthcare staff',
    'Healthcare Professional Staff',
    'Healthcare Professional',
    'Healthcare Services Staff',
    'Support Healthcare Staff',
    'Support Specialist',
    'Laser Technician',
    'Clincal Manager',
    'Surgical coordinator',
    'Scrub Tech',
    'Laser Tech',
    'Medical coordinator',
    'Patient Care Coordinator',
    'Patient care Coordinator',
    'Patient coordinator',
    'Woundcare Specialist',
    'Certified Chief Surgical Technician Bariatrics Colorectal General & Robotic Surgery',
    'Certified Dermatology Technician',
    'Dermatology Clinical Assistant',
    'Dermatology Technician',
    'General Surgery Clinical Coordinator',
    'General Surgery Inventory Coordinator',
    'Plastic & General Surgery Coordinator',
    'Residency Program General Surgery Coordinator',
    'Primary Coordinator',
    'Medical Coordinator',
    'Positions assistant',
    'Surgical Scrub Tech',
    'TECNICA',
    'Auxiliar en enfermeria',
    'C&D',
    'CMA',
    'CNA',
    'FSC',
    'FSE',
    'FSU',
    'Field Service Engineer',
}

# Define patterns for  & Resident & Professor (these should NOT be changed)
clinical_staff_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Physician Clinical Education Consultant',
}

# Create a mask for Clinical Staff
mask_clinical_staff = df['speciality'].str.contains('|'.join(clinical_staff_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(clinical_staff_exact_matches)
mask_clinical_staff_exclusions = df['speciality'].isin(clinical_staff_exclusions)

# Final mask: Select Clinical Staff
mask_clinical_staff_final = mask_clinical_staff & ~mask_clinical_staff_exclusions

# Store the original values that will be replaced
original_clinical_staff_values = df.loc[mask_clinical_staff_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_clinical_staff_final, 'speciality'] = 'Clinical Staff'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Clinical Staff", 'green'))
print(df.loc[mask_clinical_staff_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_clinical_staff_values = df.loc[mask_clinical_staff_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Clinical Staff", "cyan"))
for original_clinical_staff_value in original_clinical_staff_values:
    print(f"✅ {original_clinical_staff_value} → Clinical Staff")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Clinical Staff:", 'red'))
print(grouped_clinical_staff_values)

# Print summary
matched_count_clinical_staff = mask_clinical_staff_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Clinical Staff: "
        f"{matched_count_clinical_staff}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Company CEO related titles

company_ceo_variants = [
    r"(?i)\bCompany\s?CEO\b",
    r"(?i)\bChief\s?Executive\s?Officer\b",
    r"(?i)\bChief\s?Executive\s?Officer\s?of\s?Company\b",
    r"(?i)\bExecutive\s?Officer\b",
    r"(?i)\bCEO\s?of\s?Company\b",
    r"(?i)\bChief\s?Officer\b",
    r"(?i)\bCEO\s?of\s?the\s?Company\b",
    r"(?i)\bPresident\s?of\s?Company\b",
    r"(?i)\bChief\s?Officer\s?Company\b",
    r"(?i)\bHead\s?of\s?Company\b",
    r"(?i)\bceo\b",
    r"(?i)\bc.e.o\b",
    r"(?i)\b(c\.?e\.?o\.?)\b",
    r"(?i)\bfounder\b",
    r"(?i)\bcoo\b",
    r"(?i)\bgm\b",
    r"(?i)\bGM\b",
    r"(?i)\bGm\b",
    r"(?i)\bCfo\b",
    r"(?i)\bCFO\b",
    r"(?i)\bcfo\b",
    r"(?i)\bGeneral Director\b",
    r"(?i)\bSenior Vice President\b",
    r"(?i)\bPresidentCeo\b",
    r"(?i)\bCso\b",
    r"(?i)\bCeoManaging Director\b",
    r"(?i)\bceo\b",
    r"(?i)\bPresident\b",
    r"(?i)\bCto\b",
    r"(?i)\bSvp\b",
    r"(?i)\bBusinessman\b",
    r"(?i)\bDeputy General Manager\b",
    r"(?i)\bInvestor\b",
    r"(?i)\bGeneral Manager Mena\b",
    r"(?i)Chief Executive",
    r"(?i)\bFounder\b",
    r"(?i)\bChief Of The Board\b",
    r"(?i)\bPrsidente\b",
    r"(?i)\bFondatrice\b",
    r"(?i)\bGeneral Managerr\b",
    r"(?i)\bChief Financial Officer\b",
    r"(?i)\bDirektur\b",
    r"(?i)\bGeneral Managing\b",
    r"(?i)\bDirecteur General\b",
    r"(?i)\bGeneral Manager Uk&I - Ea\b",
    r"(?i)\bDirectrice\b",
    r"(?i)\bVp National & Strategic Accounts\b",
    r"(?i)\bGeneral Manager Uk&I\b",
    r"(?i)\bVp\b",
    r"(?i)\bCio\b",
    r"(?i)\bPresigent\b",
    r"(?i)\bVice Secretary Insdv Indonesia\b",
    r"(?i)\bAssistant General Manager\b",
    r"(?i)\bPresidente\b",
]

# Exact matches that should be updated
company_ceo_exact_matches = {
    'Company CEO',
    'Chief Executive Officer',
    'Chief Executive Officer of Company',
    'Executive Officer',
    'CEO of Company',
    'Chief Officer',
    'CEO of the Company',
    'President of Company',
    'Chief Officer Company',
    'Head of Company',
    'PresidentCeo',
    'G M',
    'GM',
    'Gm',
    'gm',
    'gM',

    # Common misspellings for Company CEO titles
    'Chief Executve Officer',
    'Compnay CEO',
    'Chif Executive Officer',
    'Chief Excutive Officer',
    'Ceo of Company',
    'Chief Excutive Offcer',
    'Comapny CEO',
    'Executive Offcer',
    'Executve Officer',
    'Chief Excutive Ocer',
    'CEO of Comapny',
    'Chief Excutive Ofcer',

    # Case-related errors for Company CEO titles
    'COMPANY CEO',
    'CHIEF EXECUTIVE OFFICER',
    'EXECUTIVE OFFICER',
    'CEO OF COMPANY',
    'CHIEF OFFICER',
    'CEO OF THE COMPANY',
    'PRESIDENT OF COMPANY',
    'CHIEF OFFICER COMPANY',
    'HEAD OF COMPANY',
    'chief executive officer',
    'executive officer',
    'company ceo',
    'Assistant Director',

    # Variants of Company CEO for different types in Spanish
    'CEO de la Compañía',
    'Director Ejecutivo',
    'Presidente de la Compañía',
    'Director General',
    'Director Ejecutivo de la Compañía',
    'Jefe de la Compañía',
    'Jefe Ejecutivo de la Compañía',
    'Presidente Ejecutivo',
    'Director General de la Compañía',
    'Director de la Compañía',

    # Other possible variations (Including Doctor forms, Specialist forms)
    'Managing Director',
    'Executive Director',
    'President of the Company',
    'Principal Executive Officer',
    'Corporate CEO',
    'Chief Operating Officer',
    'Company Head',
    'Corporate President',
    'President',
    "Presidnt",
    "Presdent",
    "Presdient",
    "Preisdent",
    "Presidentt",
    "Preident",
    "Preeident",
    "Presdnt",
    "Ptesident",
    'Vice President',
    'Vp',
    'VP',
    'vp',
    'Cso',
    'CSO',
    'cso',
    'CeoManaging Director',
    'CeoChairmanMd',
    'Business Director',
    'Deputy Director',
    'Regional Director',
    'Business Head',
    'Chairman',
    'CeoMd',
    'FounderCeo',
    'Chairman Sylhet District',
    'Cofounder',
    'Chairman Of Board',
    'Chairman Of The Board',
    'Member - Board Of Directors',
    'CeoPresident',
    'Chief Business Officer',
    'PresidentCeoOwner',
    'Vice Chairman',
    'Chief International Officer',
    'CeoFounder',
    'Dirigeant',
    'Chief Technology Officer',
    'Physician associateCEO',
    'PresidentFounder',
    'CVO',
    'Cvo',
    'cvo',
    'Chief Revenue Officer',
    'Chief Executive Officer & Premier Partners Dentist',
    'CBDO',
    'EVP',
}

# Define patterns for owner (these should NOT be changed)
company_ceo_exclusions = [
    r"(?i)\b(owner|co-owner|owner-manager|coowner|owner-|assistant)\b",
    r"(?i)\bAssistant\b",
    r"(?i)\bSurgeon\b",
    r"(?i)\bPhysician\b",
    r"(?i)\bRN\b",
    r"(?i)\bRn\b",
    r"(?i)\bRegistered Nurse\b",
    r"(?i)\bARNP\b",
    r"(?i)\bLPN\b",
    r"(?i)\bNP\b",
    r"(?i)\bAdvanced Registered Nurse Practitioner\b",
    r"(?i)\bNurse\b",
    r"(?i)\bMarketing\b",
    r"(?i)\bSales\b",
    r"(?i)\bDentist\b",
    r"(?i)\bDermatologist\b",
    r"(?i)\bCosmetologist\b",
]

# Join exclusion patterns into a single regex string
exclusions_pattern = '|'.join(company_ceo_exclusions)

# Create a mask for Company CEO
mask_company_ceo = df['speciality'].str.contains('|'.join(company_ceo_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(company_ceo_exact_matches)

# Apply exclusions using str.contains() instead of .isin()
mask_company_ceo_exclusions = df['speciality'].str.contains(exclusions_pattern, case=False, na=False, regex=True)

# Final mask: Select Company CEO
mask_company_ceo_final = mask_company_ceo & ~mask_company_ceo_exclusions

# Store the original values that will be replaced
original_company_ceo_values = df.loc[mask_company_ceo_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_company_ceo_final, 'speciality'] = 'Company CEO'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Company CEO", 'green'))
print(df.loc[mask_company_ceo_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_company_ceo_values = df.loc[mask_company_ceo_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Company CEO", "cyan"))
for original_company_ceo_value in original_company_ceo_values:
    print(f"✅ {original_company_ceo_value} → Company CEO")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Company CEO:", 'red'))
print(grouped_company_ceo_values)

# Print summary
matched_count_company_ceo = mask_company_ceo_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Company CEO: "
        f"{matched_count_company_ceo}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Psychologist related titles

psychologist_variants = [
    # Standard Titles & Variants
    r"(?i)\bPsychologist\b",
    r"(?i)\bClinical Psychologist\b",
    r"(?i)\bLicensed Psychologist\b",
    r"(?i)\bPsychology Specialist\b",
    r"(?i)\bBehavioral Psychologist\b",
    r"(?i)\bMental Health Psychologist\b",

    # Misspellings & Typographical Errors
    r"(?i)\bPshychologist\b",
    r"(?i)\bPsychologst\b",
    r"(?i)\bPsycholigist\b",
    r"(?i)\bPsycholgist\b",
    r"(?i)\bPsycologist\b",
    r"(?i)\bPsychollogist\b",
    r"(?i)\bPsychologgist\b",
    r"(?i)\bPsychologisst\b",
    r"(?i)\bPsichologist\b",

    # Case Variations
    r"(?i)\bPSYCHOLOGIST\b",
    r"(?i)\bpsychologist\b",
    r"(?i)\bPsYcHoLoGiSt\b",

    # Spanish Variants
    r"(?i)\bPsicólogo\b",
    r"(?i)\bPsicóloga\b",
    r"(?i)\bDoctor en Psicología\b",
    r"(?i)\bEspecialista en Psicología\b",
    r"(?i)\bMédico Psicólogo\b",
    r"(?i)\bPsicólogo Clínico\b",
    r"(?i)\bPsicólogo de la Salud\b",

    # Hybrid Spanish-English Variants
    r"(?i)\bPsicologyst\b",
    r"(?i)\bPsychologa\b",

    # Other Possible Variations (Doctor Forms, Specialist Forms)
    r"(?i)\bConsultant Psychologist\b",
    r"(?i)\bSenior Psychologist\b",
    r"(?i)\bLead Psychologist\b",
    r"(?i)\bForensic Psychologist\b",
    r"(?i)\bChild Psychologist\b",
    r"(?i)\bGeriatric Psychologist\b",
    r"(?i)\bNeuropsychologist\b",
    r"(?i)\bPsychology\b",
    r"(?i)\bChild & Adolescent Neuropsichiatry\b",
]

# Exact matches that should be updated
psychologist_exact_matches = {
    "Psychologist",
    "Clinical Psychologist",
    "Licensed Psychologist",
    "Psychology Specialist",
    "Behavioral Psychologist",
    "Mental Health Psychologist",
    "Pshychologist",
    "Psychologst",
    "Psycholigist",
    "Psycholgist",
    "Psycologist",
    "Psychollogist",
    "Psychologgist",
    "Psychologisst",
    "Psichologist",
    "Psicólogo",
    "Psicóloga",
    "Doctor en Psicología",
    "Especialista en Psicología",
    "Médico Psicólogo",
    "Psicólogo Clínico",
    "Psicólogo de la Salud",
    "Psicologyst",
    "Psychologa",
    "Consultant Psychologist",
    "Senior Psychologist",
    "Lead Psychologist",
    "Forensic Psychologist",
    "Child Psychologist",
    "Geriatric Psychologist",
    "Neuropsychologist",
}

# # Define patterns (these should NOT be changed)
# psychologist_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

psychologist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Psychologist
mask_psychologist = df['speciality'].str.contains('|'.join(psychologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(psychologist_exact_matches)

# mask_psychologist_exclusions = df['speciality'].str.contains(psychologist_exclusions, case=False, na=False, regex=True)
mask_psychologist_exclusions = df['speciality'].isin(psychologist_exclusions)

# Final mask: Select Psychologist
mask_psychologist_final = mask_psychologist & ~mask_psychologist_exclusions

# Store the original values that will be replaced
original_psychologist_values = df.loc[mask_psychologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_psychologist_final, 'speciality'] = 'Psychologist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Psychologist", 'green'))
print(df.loc[mask_psychologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_psychologist_values = df.loc[mask_psychologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Psychologist", "cyan"))
for original_psychologist_value in original_psychologist_values:
    print(f"✅ {original_psychologist_value} → Psychologist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Psychologist:", 'red'))
print(grouped_psychologist_values)

# Print summary
matched_count_psychologist = mask_psychologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Psychologist: "
        f"{matched_count_psychologist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Plastic Surgeon related titles

plastic_surgeon_variants = [
    # Standard Titles & Variants
    r"(?i)\bCosmetic and Plastic Surgeon\b",
    r"(?i)\bSpecialistinplasticsurgery\b",

    # Misspellings & Typographical Errors
    r"(?i)\bPlasic Surgeon\b",
    r"(?i)\bPlastic Surgon\b",
    r"(?i)\bPlastic Surgen\b",
    r"(?i)\bPlstic Surgeon\b",

    # Spanish Variants
    r"(?i)\bCirujano Plástico\b",
    r"(?i)\bCirujano Plástico Especialista\b",

    # Other Possible Variations
    r"(?i)\bCosmetic and Plastic Surgery Specialist\b",
    r"(?i)\bPlastic Surgery Expert\b",
    r"(?i)\bConsultantPlastic Surgeon\b",
    r"(?i)\bConsultantPlastic Surgery\b",
    r"(?i)\bInjectablesPlastic Surgery\b",
    r"(?i)\bPlaatic Surgery\b",
    r"(?i)\bPlasti Surgeon\b",
    r"(?i)\bPlastnic Surgeon\b",
    r"(?i)\bPlastic Surgical\b",
    r"(?i)\bPlastic Surgeons\b",
    r"(?i)\bChirurgia Plastica\b",
    r"(?i)\bSurgery Plastic\b",
    r"(?i)\bCirurgiao Plastico\b",
    r"(?i)\bChirurgie Plastica\b",
    r"(?i)\bPlasticsurgeon\b",
    r"(?i)\bConsultant Plastic Surgery\b",
    r"(?i)\bHead Of Plastic Surgery\b",
    r"(?i)\bPlastic Surgery\b",
    r"(?i)\bSpecialist Plastic Surgery\b",
    r"(?i)\bPlastic Surgeron\b",
    r"(?i)\bChief - Section Of Plastic Surgery\b",
    r"(?i)\bBurns & Plastic Surgery\b",
    r"(?i)\bPlastic Surgery Consultant\b",
    r"(?i)\bPlastic Surgery Specialist\b",
    r"(?i)\bDirector Plastic Surgery\b",
    r"(?i)\bPlasticAesthetic Surgery\b",
    r"(?i)\bDirector Advanced Plastic Surgery Centre\b",
    r"(?i)\bPlatic Surgeon\b",
    r"(?i)\bOphthalmic Plastic Surgery\b",
    r"(?i)\bPlastic Surgery Non Board Certified\b",
    r"(?i)\bPlastic Surgery Md Phd\b",
    r"(?i)\bEnt Plastic Surgery\b",
    r"(?i)\bDirector General & Hd Of Plastic Surgery Dept\b",
    r"(?i)\bChief - Plastic Surgery\b",
    r"(?i)\bChief Pediatric Plastic Surgery\b",
    r"(?i)\bDirector Of Injectables Bucky Plastic Surgery\b",
    r"(?i)\bChirurgien Plasticien\b",
    r"(?i)\bChirurgie Plastica\b",
    r"(?i)\bCirurgia Plastica\b",
    r"(?i)\bCirujano Plastico\b",
    r"(?i)\bChirurgie Plastique\b",
    r"(?i)\bMedica Cirurgia Plastica\b",
    r"(?i)\bMedico Chirurgo Plastico\b",
    r"(?i)\bPlastic Chirurgie\b",
    r"(?i)\bCirujano Plasico\b",
    r"(?i)\bPlasticheskii Khirurg\b",
    r"(?i)\bConsultant Plastic Surgeon\b",
    r"(?i)\bBeverly Hills Plastic Surgeon\b",
]

# Exact matches that should be updated
plastic_surgeon_exact_matches = {
    "Plastic Surgeon",
    "Cosmetic and Plastic Surgeon",
    "Plasic Surgeon",
    "Plastic Surgon",
    "Plastic Surgen",
    "Plstic Surgeon",
    "Cirujano Plástico",
    "Cirujano Plástico Especialista",
    "Aesthetic Surgery Specialist",
    "Cosmetic and Plastic Surgery Specialist",
    "Plastic Surgery Expert",
    "Plastic",
    "Plastics",
    'Plastic Surgery',
    'Plastic Surgeon',
    'Plasticsurgeon',
    'Plasic Surgeon',
    'PlastiC Surgeon',
    'Plastic Surgery Specialist',
    'Plastic Surgeron',
    'Platic Surgeon',
    'PlastiC Surgeon',
    'Plastc Surgeon',
    'Plastc Surgeron',
    'Plasticsurgen',
    'Plasic Surgeon',
    'Plastic Surgeion',
    'plasticsurgeon',
    'Plasticsurgen',
    'Plastic Surgeon',
    'Plastic surgeon',
    'plastic surgeon',
    'PlasticSurgeon',
    'Plasticsurgeon',
    'Cirugia Plastica',
    'Medical Director & Plastic Surgeon',
    'Plastic Surgeon & Medical Director',
    'Plastic Surgeon - Antiaging Practicioner',
    'Md Plastics',
    'Chirurgia Palastica',
    'Cirujana Plastica',
    'Board Certified Plastic Surgeon',
    'Medical Director Plastic Surgeon',
}

# # Define patterns (these should NOT be changed)
plastic_surgeon_exclusions = r'\b(?:Aesthetic)|(?:Facial)|(?:Maxillofacial)|(?:Resident)|(?:Professor)|(?:Sales)\b'

# plastic_surgeon_exclusions = {
#     'Resident', 'resident', 'student', 'Trainee', 'Resident Doctor', 'Resident ooPhysician',
#     'Intern', 'intern', 'Medical Intern', 'Fellow', 'fellow', 'Clinical Fellow', 'Medical Student',
#     'Clinical Trainee', 'Trainee Doctor', 'Trainee Physician', 'Junior Doctor', 'Postgraduate Trainee',
#     'Aesthetic Fellow', 'Aesthetic Trainee', 'Aesthetic Medicine Fellow', 'Aesthetic Resident'
# }

# Create a mask for Plastic Surgeon
mask_plastic_surgeon = df['speciality'].str.contains('|'.join(plastic_surgeon_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(plastic_surgeon_exact_matches)

mask_plastic_surgeon_exclusions = df['speciality'].str.contains(plastic_surgeon_exclusions, case=False, na=False, regex=True)
# mask_plastic_surgeon_exclusions = df['speciality'].isin(plastic_surgeon_exclusions)

# Final mask: Select Plastic Surgeon
mask_plastic_surgeon_final = mask_plastic_surgeon & ~mask_plastic_surgeon_exclusions

# Store the original values that will be replaced
original_plastic_surgeon_values = df.loc[mask_plastic_surgeon_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_plastic_surgeon_final, 'speciality'] = 'Plastic Surgeon'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Plastic Surgeon", 'green'))
print(df.loc[mask_plastic_surgeon_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_plastic_surgeon_values = df.loc[mask_plastic_surgeon_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Plastic Surgeon", "cyan"))
for original_plastic_surgeon_value in original_plastic_surgeon_values:
    print(f"✅ {original_plastic_surgeon_value} → Plastic Surgeon")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Plastic Surgeon:", 'red'))
print(grouped_plastic_surgeon_values)

# Print summary
matched_count_plastic_surgeon = mask_plastic_surgeon_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Plastic Surgeon: "
        f"{matched_count_plastic_surgeon}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for andrologist related titles
andrologist_variants = [
    r"(?i)\bAndrologist\b",
    r"(?i)\bAndrology Specialist\b",
    r"(?i)\bAndrology Doctor\b",
    r"(?i)\bAndrology Consultant\b",
    r"(?i)\bAndrology Expert\b",
    r"(?i)\bMale Reproductive Specialist\b",
    r"(?i)\bMen’s Health Specialist\b",
    r"(?i)\bMale Fertility Doctor\b",
    r"(?i)\bAndrology Researcher\b",
]

# Exact matches that should be updated
andrologist_exact_matches = {
    'Andrologist',
    'Andrology Specialist',
    'Andrology Doctor',
    'Andrology Consultant',
    'Andrology Expert',
    'Male Reproductive Specialist',
    'Men’s Health Specialist',
    'Male Fertility Doctor',
    'Andrology Researcher',
    'Clinical Andrologist',
    'Reproductive Andrologist',
    'Surgical Andrologist',
    'Medical Andrologist',

    # Case-related errors
    'andrologist',
    'ANDROLOGIST',
    'AnDrOlOgIsT',
    'ANDROLOgist',
    'aNDROLOGIST',

    # Common misspellings
    'Androlgist',
    'Androlgist',
    'Andorlogist',
    'Andoroligst',
    'Andorologist',
    'Androlgist',
    'Andorlogy Specialist',
    'Androlagist',
    'Androlegist',

    # Spanish-related exclusions
    'Andrólogo',
    'Especialista en Andrología',
    'Médico Andrólogo',
    'Consultor en Andrología',
    'Investigador en Andrología',
    'Experto en Andrología',
    'Doctor en Andrología',
    'Especialista en Salud Masculina',
    'Especialista en Reproducción Masculina',
}

# Define patterns for  & Resident & Professor (these should NOT be changed)
andrologist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Andrologist
mask_andrologist = df['speciality'].str.contains('|'.join(andrologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(andrologist_exact_matches)
mask_andrologist_exclusions = df['speciality'].isin(andrologist_exclusions)

# Final mask: Select Andrologist
mask_andrologist_final = mask_andrologist & ~mask_andrologist_exclusions

# Store the original values that will be replaced
original_andrologist_values = df.loc[mask_andrologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_andrologist_final, 'speciality'] = 'Andrologist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Andrologist", 'green'))
print(df.loc[mask_andrologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_andrologist_values = df.loc[mask_andrologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Andrologist", "cyan"))
for original_andrologist_value in original_andrologist_values:
    print(f"✅ {original_andrologist_value} → Andrologist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Andrologist:", 'red'))
print(grouped_andrologist_values)

# Print summary
matched_count_andrologist = mask_andrologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Andrologist: "
        f"{matched_count_andrologist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

acupuncturist_exact_matches = {
    'Acupuncturist',
    'Licensed Acupuncturist',
    'Certified Acupuncturist',
    'Acupuncture Practitioner',
    'Licensed Acupuncture Practitioner',
    'Traditional Chinese Medicine Acupuncturist',
    'Traditional Chinese Medicine Practitioner',
    'TCM Acupuncturist',
    'TCM Practitioner',
    'Acupuncture Specialist',
    'Acupuncture Therapist',
    'Acupuncture Doctor',
    'Acupuncture Specialist Doctor',
    'Doctor of Acupuncture',
    'Doctor of Traditional Chinese Medicine',
    'Doctor of Acupuncture and Oriental Medicine',
    'Doctor of Oriental Medicine',
    'Licensed Acupuncture Doctor',
    'Acupuncture and Oriental Medicine Practitioner',
    'Acupuncture and Herbal Medicine Specialist',
    'Acupuncturist MD',
    'Acupuncturist with Oriental Medicine',
    'Licensed Practitioner of Acupuncture',
    'Acupuncturist Doctor',
    'Acupuncturist and Herbalist',
    'Acupuncture Therapist MD',
    'Acupuncturist (TCM)',
    'Acupuncture Therapy Specialist',
    'Licensed Chinese Medicine Practitioner',
    'Chinese Medicine Acupuncturist',
    'Acupuncture and Acupressure Specialist',
    'Licensed Acupuncture and Herbal Medicine Practitioner',
    'Acupuncture and TCM Doctor',
    'Chinese Medicine Acupuncturist',
    'Acupuncturist Practitioner (TCM)',
    'Acupuncture and TCM Specialist',
    'Acupuncturist L.Ac',
    'Acupunctureist',
    'Acupunture Speciallist',
    'Acuprunture Specialist',
    'Acupunturist Specialist',
    'Acupuncturist Dipl. Ac',
    'Acupuncturist D.O.M.',
    'Doctor of Oriental Medicine Acupuncturist',
    'Acupunture',
    'Acupunturist',
    'Acupruncturist',
    'Acuponcturist',
    'Acupuncturist',
    'Acupuntureist',
    'Acupunturist doctor',
    'acupuncturist with oriental medicine',
    'Acupunture and oriental medicine',
    'Certified Traditional Chinese Medicine Acupuncturist',
    'Chinese medicine acupunturist',
    'Chinese medcine acupuncturist',
    'Acupuncturist TCM',
    'acupuncture practioner',
    'Doctor of Acupuncture and Chinese Medicine',
    'Licensed Acupuncture and Chinese Medicine Practitioner',
    'Certified Acupuncture Specialist',
    'TCM acupuncturist practitioner',
    'TCM acupuncture practitioner',
    'AcupuncturIst Md',
    'AcupuncturIst D.o.m',
    'Tcm acupuncturist',
    'tcM acupuncturist',
    'acupuncturist (tcm)',
    'tcm practicioner',
    'tcm practioner',
    'acupuncture',
    'acupuncturist',
    'aCuPuncturist',
    'aCUPUNCTURIST',
    'acupuncturIst',
    'Acupunture therapist',
    'acupunture practitioner',
    'acupuncture therapiest',
    'Doctor of acupuncture and chinese medicine',
    'tcm acupunturist',
    'acupuncturist and herbalist',
    'AcupuncturIst and HerbalIst',
    'Acupuncurist doctor',
# Variations for common typos and case mistakes
}

# Apply transformation and track changes
mask_acupuncturist = df['speciality'].str.contains('Acupuncturist', case=False, na=False, regex=True) | \
                     df['speciality'].isin(acupuncturist_exact_matches)

# Store the original values that will be replaced
original_acupuncturist_values = df.loc[mask_acupuncturist, 'speciality'].unique()

# Update the speciality column
df.loc[mask_acupuncturist, 'speciality'] = 'Acupuncturist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Acupuncturist", 'green'))
print(df.loc[mask_acupuncturist, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_acupuncturist_values = df.loc[mask_acupuncturist, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Acupuncturist", "cyan"))
for original_acupuncturist_value in original_acupuncturist_values:
    print(f"✅ {original_acupuncturist_value} → Acupuncturist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Acupuncturist:", 'red'))
print(grouped_acupuncturist_values)

# Print summary
matched_count_acupuncturist = mask_acupuncturist.sum()

# Print results
print(colored(f"\nTotal values matched and changed (Stage 1) to Acupuncturist: {matched_count_acupuncturist}", 'red'))



import pandas as pd
import numpy as np
import re
from nltk.corpus import stopwords
import unicodedata
import sys
import os
from termcolor import colored

# Remove the original_speciality column as no longer needed
df = df.drop(columns = ['original_speciality'])

# Declare the list of values available in main_title table
list_of_job_titles = [
    'Acupuncturist',
    'Advanced Registered Nurse Practitioner (ARNP)',
    'Aesthetic & Anti-aging Physician',
    'Aesthetic & Plastic Surgeon',
    'Aesthetic Physician',
    'Alternative Medicine Physician',
    'Anaesthesist',
    'Anatomist',
    'Andrologist',
    'Angiologist Phlebologist',
    'Anti-aging Physician',
    'Biologist Biochemist',
    'Biomedical Physician',
    'Cardiologist',
    'Clinic Manager',
    'Clinical Staff',
    'Company CEO',
    'Company Manager',
    'Company Medical Staff',
    'Company Representative',
    'Consultant',
    'Cosmetic Dermatologist',
    'Cosmetic Surgeon',
    'Cosmetologist MD',
    'Cytologist',
    'Dental Assistant',
    'Dental Surgeon',
    'Dentist',
    'Dermatologic Surgeon',
    'Dermatologist',
    'Distributor',
    'Employee',
    'Endocrinologist',
    'ENT Ear Nose Throat Physician',
    'Esthetician Cosmetician',
    'Event Manager',
    'Facial Plastic Surgeon',
    'General Physician',
    'General Surgeon',
    'Genetician',
    'Geriatrist',
    'Gynecologist Obstetrician',
    'Hair Transplant Surgeon',
    'Head of department',
    'Hematologist',
    'Hospital Manager',
    'Immunologist',
    'Influencer',
    'Internal Medicine',
    'Laboratory Medicine',
    'Lawyer',
    'Legal Counsel',
    'Licensed Practical Nurse (LPN)',
    'Marketing Manager',
    'Maxillofacial Surgeon',
    'Medical Advisor',
    'Medical Spa Manager',
    'Neonatologist',
    'Neurologist',
    'Neurophysiologist',
    'Non medical Staff',
    'Nurse Practitioner (NP)',
    'Nutritionist',
    'Oculoplastic Surgeon',
    'Oncologist',
    'Ophthalmologist',
    'Oral and Maxillofacial Surgeon',
    'Orthodontist',
    'Orthopedist',
    'Owner',
    'Pathologist',
    'Pediatrician',
    'Pharmacist',
    'Pharmacologist',
    'Photographer',
    'Physician Assistant',
    'Physiotherapist',
    'Plastic and Reconstructive Surgeon',
    'Plastic Surgeon',
    'Practice Manager',
    'Press',
    'Product Project Manager',
    'Professor Teacher',
    'Psychiatrist',
    'Psychologist',
    'Psychotherapist',
    'Radiologist',
    'Registered Nurse (RN)',
    'Research Scientist',
    'Rheumatologist',
    'Sales',
    'Sports Medicine Physician',
    'Stomatologist',
    'Student Resident Fellow',
    'Trichologist',
    'Urologist',
    'Vascular Surgeon',
    'ENT Head and Neck Specialist', # FOR EUROGIN
	'General Manager', # Old Value and doesnt exists in main title
	'Nurse' # Old Value and doesnt exists in main title
]

# Filter non-matching job titles after processing
not_transformed_job_titles_stage1 = pd.DataFrame(df[~df['speciality'].isin(list_of_job_titles)])

# GroupBy to count how many records were not transformed to business format
grouped_not_transformed_job_titles = not_transformed_job_titles_stage1['speciality'].value_counts()

# Print non-matching titles and their count
print(colored("\nRemaining Job Titles that have not been transformed to business value format after processing:", 'red'))
print(tab(grouped_not_transformed_job_titles.head(100).reset_index(), headers=['Speciality', 'Count'], tablefmt='psql'))

# Print count of non-matching job titles
print(colored(f"Count of non-matching job titles after processing: {not_transformed_job_titles_stage1.shape[0]}", 'red'))

# Filtering logic - Job Title Is Other, Unknown, an integer or not relevant after processing
df_records_job_title_other = not_transformed_job_titles_stage1[
    not_transformed_job_titles_stage1['speciality'].astype(str).str.strip().isin(['Other', 'Unknown', 'None']) |  # Remove specified values
    not_transformed_job_titles_stage1['speciality'].astype(str).str.match(r'^\d+$') |  # Remove numbers only
    not_transformed_job_titles_stage1['speciality'].astype(str).str.match(r'^[A-Za-z]$')  # Remove single letters
]

# Print records were Job Title Is Other, Unknown, an integer or not relevant after processing
print(colored("\nJob Title Is Other, Unknown, an integer or not relevant:", 'red'))
print(tab(df_records_job_title_other.head(10).reset_index(),  headers='keys', tablefmt='psql'))

# Print count of records were Job Title Is Other, Unknown, an integer or not relevant after processing
print(colored(f"\nCount Job Title Is Other, Unknown, an integer or not relevant: {df_records_job_title_other.shape[0]}", 'red'))

################################################## IMPORTANT ###########################################################
################################################## IMPORTANT ###########################################################
################################################## IMPORTANT ###########################################################
################################################## IMPORTANT ###########################################################

# Comment / Uncomment the below part if you do not wish to implement ML model to handle the not transformed job title values

# Tips: For Data coming from internal sources - Uncomment the below
# Tips: If you handle large volume of data (both internal and external source) comment the below
# Tips: Uncomment the below when you handle 100k to 250k records

# Version Note: 0.1.10

# print(colored("\nRecords were job_title has been transformed:", 'red'))
# # Filter only transformed records (those that were successfully replaced)
# transformed_job_titles = df[df['speciality'].isin(list_of_job_titles)].copy()
# print(tab(transformed_job_titles.head(10), headers='keys', tablefmt='psql'))
#
# # create a copy for concat operation during ML Stages
# df_transformed = transformed_job_titles.copy()
# print(colored("\nFinal Dataframe at the end of processing Job Titles Pipeline", "red", attrs=["bold"]))
# print(tab(df_transformed.head(10), headers='keys', tablefmt='psql'))
#
# # Print count of records with transformed job titles
# print(colored(f"\nCount of transformed job titles after Processing: {transformed_job_titles.shape[0]}", 'red'))
#
# # Get the row counts of df_test and df
# count_not_transformed_job_titles = len(not_transformed_job_titles)
#
# # For Programmer's Review
# if count_not_transformed_job_titles > 0:
#     print(colored(
#         "\nThe count of not transformed job titles is greater than 0. The programmer needs to review these values "
#         "\nbefore proceeding with further execution.", 'red'))
#
#     # Print the count for debugging
#     print(f"Count of not transformed job titles: {count_not_transformed_job_titles}")
#
#     # Exit the pipeline
#     sys.exit("Exiting pipeline as programmer needs to review the non-transformed job titles.")
# else:
#     print(colored(
#         "\nThe count of transformed job titles is 0. The pipeline proceeds to the next stages.", 'green'))
#
#     # Print the count for confirmation
#     print(f"Count of not transformed job titles: {count_not_transformed_job_titles}")
#
# # Concat Frames for next stage of execution
# df = pd.concat([
#     df,
#     not_transformed_job_titles,
#     df_other,
#     df_CWS
# ], ignore_index=True)
#
# print(colored("\nFinal Dataframe at the end of processing Job Titles Pipeline that will be sent for further execution", "red", attrs=["bold"]))
# print(tab(df_transformed.head(10), headers='keys', tablefmt='psql'))
#
# # Print count of records with transformed job titles
# print(colored(f"\nCount of records at the end of processing Job Titles Pipeline that will be sent for further execution: {transformed_job_titles.shape[0]}", 'red'))
#
# # Testing
#
# # Get the row counts of df_test and df
# count_df_test = len(df_test)
# count_df = len(df)
#
# # Check if the counts match
# if count_df_test == count_df:
#     print(colored(
#         "\nThe count of source dataframe before processing job title transformation stages and after processing job "
#         "\ntitle transformation pipeline remains same/equal. Thus, the pipeline will proceed its execution.", 'green'))
# else:
#     print(colored(
#         "\nThe count of source dataframe before processing job title transformation stages and after processing job "
#         "\ntitle transformation pipeline are not same/equal. Thus, the pipeline will stop its execution, you need to "
#         "\nreview the pipeline.", 'red'))
#
#     # Print the counts
#     print(f"Count of df_test: {count_df_test}")
#     print(f"Count of df: {count_df}")
#
#     # Exit the program if the counts don't match
#     sys.exit("Exiting pipeline due to mismatched record counts.")

################################################## ML TECHNIQUE ########################################################

# Tips: For Data coming from External sources - Uncomment the below
# Tips: If you handle large volume of data (both internal and external source) Uncomment the below
# Tips: Uncomment the below when you handle 1M to 5M records

print(colored("\nRecords were job_title has been transformed after processing STAGE 1:", 'red'))
# Filter only transformed records (those that were successfully replaced)
transformed_job_titles_stage1 = df[df['speciality'].isin(list_of_job_titles)].copy()

# create a copy for concat operation during ML Stages
df_transformed_stage1 = transformed_job_titles_stage1.copy()
print(colored("\nFinal Transformed Dataframe at the end of processing Job Titles Pipeline STAGE 1", "red", attrs=["bold"]))
print(tab(df_transformed_stage1.head(10), headers='keys', tablefmt='psql'))

# Print count of records with transformed job titles after processing STAGE 1
print(colored(f"\nCount of transformed job titles after Processing STAGE 1: {transformed_job_titles_stage1.shape[0]}", 'red'))

# ML / AI model

from sentence_transformers import SentenceTransformer, InputExample, losses
from torch.utils.data import DataLoader
import faiss
import torch
from sklearn.model_selection import train_test_split
from sentence_transformers import losses

# ML / AI model

# Load SBERT model
model = SentenceTransformer('all-MiniLM-L6-v2')

# Convert job titles into SBERT embeddings
title_embeddings = model.encode(list_of_job_titles)

# Initialize FAISS index
index = faiss.IndexFlatL2(title_embeddings.shape[1])
index.add(np.array(title_embeddings))  # Store embeddings

# Extract job titles for AI processing
unmatched_titles = not_transformed_job_titles_stage1["original_speciality"].tolist()

# Convert to SBERT embeddings (batch processing)
batch_size = 30  # Process in batches to handle large data efficiently
for i in range(0, len(unmatched_titles), batch_size):
    batch_titles = unmatched_titles[i : i + batch_size]
    batch_embeddings = model.encode(batch_titles)  # Convert to embeddings

    # Search closest match in FAISS
    _, match_indexes = index.search(np.array(batch_embeddings), 1)

    print(len(batch_titles), len(match_indexes[:, 0]))
    print(match_indexes.shape)

    # Ensure index alignment before assigning values
    not_transformed_job_titles_stage1.iloc[i: i + len(batch_titles),
    not_transformed_job_titles_stage1.columns.get_loc("speciality")] = [
        list_of_job_titles[idx] for idx in match_indexes[:, 0]  # Ensure correct index mapping
    ]

# Print results
print(colored("\nNot Transformed Job Titles After ML / AI Processing", "green", attrs=["bold"]))
grouped = not_transformed_job_titles_stage1.groupby(['original_speciality', 'speciality']).size().reset_index(name='count')
for idx, row in grouped.iterrows():
    print(f"✅ {row['original_speciality']} → {row['speciality']} ({row['count']} times)")

# Filter non-matching job titles after processing STAGE 2
not_transformed_job_titles_stage2 = pd.DataFrame(not_transformed_job_titles_stage1[~not_transformed_job_titles_stage1['speciality'].isin(list_of_job_titles)])

# Get the row counts of df_test and df
count_not_transformed_job_titles_stage2 = len(not_transformed_job_titles_stage2)

# For Programmer's Review
if count_not_transformed_job_titles_stage2 > 0:
    print(colored(
        "\nThe count of not transformed job titles is greater than 0. The programmer needs to review these values "
        "\nbefore proceeding with further execution.", 'red'))

    # Print the count for debugging
    print(f"Count of not transformed job titles: {count_not_transformed_job_titles_stage2}")

    # Exit the pipeline
    sys.exit("Exiting pipeline as programmer needs to review the non-transformed job titles.")
else:
    print(colored(
        "\nThe count of transformed job titles is 0. The pipeline proceeds to the next stages.", 'green'))

    # Print the count for confirmation
    print(f"Count of not transformed job titles: {count_not_transformed_job_titles_stage2}")


# Function to check if the programmer has reviewed the output values
def check_review():
    response = input(colored(
        "\nHave you reviewed the output values (Original Speciality, Speciality) that have been transformed using "
        "\nSentenceTransformer and faiss? (yes/no): ", 'yellow')).strip().lower()

    if response == 'yes':
        print(colored("\nProceeding to concat and executing the further pipeline...", 'green'))
        return True
    elif response == 'no':
        print(colored(
            "\nPlease review the output values (Original Speciality, Speciality) that have been transformed using "
            "\nSentenceTransformer and faiss.", 'red'))
        exit()  # Exit the program
    else:
        print(colored("\nInvalid input. Please enter 'yes' or 'no'.", 'red'))
        return check_review()  # Ask again until a valid response is received


df_transformed_stage2 = not_transformed_job_titles_stage1.copy()

# Check if the programmer has reviewed the output values
if check_review():
    # Assuming df, not_transformed_job_titles, and df_other are already defined
    df = pd.concat([df_transformed_stage1, df_transformed_stage2, df_other], ignore_index = True)
    print(colored("\nConcatenation complete, pipeline execution can continue.", 'green'))

# # Concat Frames for next stage of execution
# df = pd.concat([
#     df,
#     df_other
# ], ignore_index=True)

print(colored("\nFinal Dataframe at the end of processing Job Titles Pipeline that will be sent for further execution", "red", attrs=["bold"]))
print(tab(df.head(10), headers='keys', tablefmt='psql'))

# Print count of records with transformed job titles
print(colored(f"\nCount of records at the end of processing Job Titles Pipeline that will be sent for further execution: {df.shape[0]}", 'red'))

# Testing

# Get the row counts of df_test and df
count_df_test = len(df_test)
count_df = len(df)

# Check if the counts match
if count_df_test == count_df:
    print(colored(
        "\nThe count of source dataframe before processing job title transformation stages and after processing job "
        "\ntitle transformation pipeline remains same/equal. Thus, the pipeline will proceed its execution.", 'green'))
else:
    print(colored(
        "\nThe count of source dataframe before processing job title transformation stages and after processing job "
        "\ntitle transformation pipeline are not same/equal. Thus, the pipeline will stop its execution, you need to "
        "\nreview the pipeline.", 'red'))

    # Print the counts
    print(f"Count of df_test: {count_df_test}")
    print(f"Count of df: {count_df}")

    # Exit the program if the counts don't match
    sys.exit("Exiting pipeline due to mismatched record counts.")

# Tip: Uncomment the below if you want to club / pool all the list of variants and exact match DICT into one list and one DICT

# # Initialize lists to hold all variants and exact matches
# all_variants = []
# all_exact_matches = []
#
# # Create a snapshot of all global variables
# global_vars = globals().copy()
#
# # Iterate over the keys in the snapshot of globals
# for var_name, var_value in global_vars.items():
#     if var_name.endswith('_variants'):  # If the variable is a _variants list
#         all_variants.extend(var_value)  # Add the variants to the all_variants list
#     elif var_name.endswith('_exact_matches'):  # If the variable is a _exact_matches set or list
#         if isinstance(var_value, dict):
#             all_exact_matches.extend(var_value.values())  # Add the exact match values to the all_exact_matches list
#         elif isinstance(var_value, (set, list)):  # Check if it's a set or a list
#             all_exact_matches.extend(var_value)  # Use extend to add the elements
#
# # Check the results
# print("All Variants:", all_variants)
# print("All Exact Matches:", all_exact_matches)





import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for General Physician related titles

general_physician_variants = [
    r"(?i)\bGeneral\s?Physician\b",
    r"(?i)\bGeneral\s?Practitioner\b",
    r"(?i)\bGP\b",
    r"(?i)\bFamily\s?Physician\b",
    r"(?i)\bFamily\s?Doctor\b",
    r"(?i)\bPrimary\s?Care\s?Physician\b",
    r"(?i)\bFamily Medicine\b",
    r"(?i)\bEpidemiologist\b",
    r"(?i)\bGeneral Practice\b",
    r"(?i)\bD O\b",
    r"(?i)\bDO\b",
    r"(?i)\bDra\b",
    r"(?i)\bMbbs\b",
    r"(?i)\bM b b s\b",
    r"(?i)\bM D\b",
    r"(?i)\bDr\b",
    r"(?i)\bDoc\b",
    r"(?i)\bMedico General\b",
    r"(?i)\bMedico Urgencias Medicas\b",
    r"(?i)\bAllergologist\b",
    r"(?i)\bGeneral Practioner\b",
    r"(?i)\bGeneral Practitioners\b",
    r"(?i)\bMedica Clinico Geral\b",
    r"(?i)\bConsulting Physician\b",
    r"(?i)\bEmergency Medicine\b",
    r"(?i)\bDocteur\b",
    r"(?i)\bInfectious Disease\b",
    r"(?i)\bDiagnosis\b",
    r"(?i)\bPodiatry\b",
    r"(?i)\bDoktor\b",
    r"(?i)\bGeneraliste\b",
    r"(?i)\bMdPrrsident\b",
    r"(?i)\bDokter\b",
    r"(?i)\bFamilymed\b",
    r"(?i)\bCritical Care Physician\b",
    r"(?i)\bMdphd\b",
    r"(?i)\bFamily Medicine\b",
    r"(?i)\bConsulting Physician\b",
    r"(?i)\bRehabilitation Specialist\b",

    # Spanish variants
    r"(?i)\bMédico\s?General\b",  # Spanish form (General Physician)
    r"(?i)\bMédico\s?de\s?Familia\b",  # Spanish form (Family Physician)
    r"(?i)\bMédico\s?Generalista\b",  # Spanish form (General Practitioner)
    r"(?i)\bMédico\s?de\s?Atención\s?Primaria\b",  # Spanish form (Primary Care Physician)

    # Other possible variations
    r"(?i)\bDoctor\s?General\b",
    r"(?i)\bDoctor\s?en\s?Medicina\s?General\b",
    r"(?i)\bBoard\s?Certified\s?General\s?Physician\b",
    r"(?i)\bGeneral\s?Medical\s?Doctor\b",
    r"(?i)\bLicensed\s?General\s?Physician\b",
    r"(?i)\bPrimary\s?Care\s?Physician\s?Specialist\b",
    r"(?i)\bCertified\s?General\s?Physician\b",
    r"(?i)\bDoctorOwner\b",
    r"(?i)\bGeneral Medicine\b",
    r"(?i)\bMedecin Generaliste\b",
    r"(?i)\bGeneral Practicioner\b",
    r"(?i)\bGeneral Physicion\b",
]

# Exact matches that should be updated
general_physician_exact_matches = {
    "General Physician",
    "General Practitioner",
    "GP",
    "Family Physician",
    "Family Doctor",
    "Primary Care Physician",
    # Spanish form matches
    "Médico General",
    "Médico de Familia",
    "Médico Generalista",
    "Médico de Atención Primaria",
    # Other possible variations
    "Doctor General",
    "Doctor en Medicina General",
    "Board Certified General Physician",
    "General Medical Doctor",
    "Licensed General Physician",
    "Primary Care Physician Specialist",
    "Certified General Physician",
    "Physician",
    "physician",
    "Physcian",
    "Phyisician",
    "Physisian",
    "Physician's",
    "Physican",
    "Phycisian",
    "Phisician",
    "Fysician",
    "Doctor",
    "Docter",
    "Dr",
    "dr",
    "Dotor",
    "Docotor",
    "Doctar",
    "Dr.",
    "dr.",
    "Dctor",
    "Doctr",
    "Docor",
    "Docr",
    'Other Physician',
    'md',
    'Md',
    'MD',
    'M D',
    'Private Clinic/Hospital',
    'Private ClinicHospital',
    'Medical Practitioner',
    'Medical Doctor',
    'Public Health Specialist',
    'Medicine',
    'General',
    'general',
    'Medecin',
    'DoctorOwner',
    'Generalist',
    'generalist',
    'Generaliste',
    'generaliste',
    'Clinic Doctor',
    'Clinic Physician',
    'Consultant Physician',
    'Family Practice',
    'Medical Director Physician',
    'Primary Care',
    'Family',
    'Medico',
    'Medica',
    'Medicine Doctor',
    'Doctor Of Medicine',
    'AestheticsGp',
    'Family MedicineAesthetics',
    'Medicina',
    'Vrach',
    'Docctor',
    'Medecin Generalist',
    'Medecine',
    'Oh Phusician',
    'Senior Physician',
    'Physical Medicine & Rehabilitation',
    'Primary  Operation',
    'Medecine Generale',
    'GpAesthetics',
    'Medico Principal',
    'Phusician',
    'Managing Practitioner',
    'Phisition',
    'Doctora',
    'Docytor',
    'ED Attending',
    'FMD',
    'MD MPH',
    'Md Ccfp Faarm',
    'Privada',
    'medico',
    'Doctor or Consultant',
    'Provider',
    'Physicisn',
    'Physician Mmg',
    'Attending Physician',
    'Emergency Physician',
    'Physician & Owner',
    'Physician - Med Onc',
    'Physician-Clinic',
    'Owner & Physician',
    'Physician & Founder',
    'Personal Physician',
    'Physician Owner',
    'Physician Partner - Hem',
    'Concierge Physician',
    'Physician & Chief Executive Officer',
    'Qp0032 - Physician',
    'Physicians',
    'Staff Physician',
    'Associate Physician',
    'Attending Physician Md',
    'Clinic Physician & Amp Ministry Medical Director Of Diabetes Care',
    'Physician & Co-Founder',
    'Physician & Employed',
    'Physician & Hospitalist',
    'Physician & Nocturnist',
    'Physician-Hospitalist Psv',
    'Senior Staff Physician',
    'Staff Physician Ii',
    'Supervising Physician',
    'Urgent Care Physician',
    'DPC',
    'EM',
    'Emergency',
    'FP',
}

# Define patterns (these should NOT be changed)
general_physician_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident ooPhysician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Family Physician Resident',
    'Emergency Medicine Resident Doctor',
    'Do You Want To Get More Sales',
    'General Medicine Student',
    'Gp Training',
    'Dr  Nurse Practitioner',
    'Doc In Maxillary Facial Surgery',
    'Practice Manager For Family Medicine Practice',
    'Personal Assistant To Dr',
    'Do Not Miss This Opportunity',
    'No Available Job I Do',
    'Senior Physician Recruiter- Emergency Medicine - Nebraska Iowa Arkansas & Kentucky',
    'Physician Recruiter--Emergency Medicine',
    'Physician Assistant - Emergency Medicine',
    'Emergency Medicine Physician Assistant',
}

# Create a mask for General Physician
mask_general_physician = df['speciality'].str.contains('|'.join(general_physician_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(general_physician_exact_matches)
mask_general_physician_exclusions = df['speciality'].isin(general_physician_exclusions)

# Final mask: Select General Physician
mask_general_physician_final = mask_general_physician & ~mask_general_physician_exclusions

# Store the original values that will be replaced
original_general_physician_values = df.loc[mask_general_physician_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_general_physician_final, 'speciality'] = 'General Physician'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: General Physician", 'green'))
print(df.loc[mask_general_physician_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_general_physician_values = df.loc[mask_general_physician_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: General Physician", "cyan"))
for original_general_physician_value in original_general_physician_values:
    print(f"✅ {original_general_physician_value} → General Physician")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with General Physician:", 'red'))
print(grouped_general_physician_values)

# Print summary
matched_count_general_physician = mask_general_physician_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to General Physician: "
        f"{matched_count_general_physician}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Stomatologist related titles

stomatologist_variants = [
    # Standard Titles & Variants
    r"(?i)\bStomatologist\b",  # General term for Stomatologist

    # Misspellings & Typographical Errors
    r"(?i)\bStomatoligist\b",  # Common misspelling
    r"(?i)\bStomotologist\b",  # Typo with swapped letters
    r"(?i)\bStomotoligist\b",  # Misspelling with typo and extra 'l'
    r"(?i)\bStomatoligist\b",  # Extra 'i' in misspelling
    r"(?i)\bStomatlogist\b",  # Misspelling with missing 'o'

    # Case Variations
    r"(?i)\bSTOMATOLOGIST\b",  # Uppercase variant
    r"(?i)\bstomatologist\b",  # Lowercase variant
    r"(?i)\bStOmAtOlOgIsT\b",  # Mixed case variant

    # Spanish Variants
    r"(?i)\bEstomatólogo\b",  # General term for Stomatologist in Spanish

    # Hybrid Spanish-English Variants
    r"(?i)\bStomatólogo Médico\b",  # Hybrid term

    # Other Possible Variations (Doctor Forms, Specialist Forms)
    r"(?i)\bOral Medicine & Pathology\b",
    r"(?i)Stomatologue",
    r"(?i)Stomatology",
]

# Exact matches that should be updated
stomatologist_exact_matches = {
    "Stomatologist",
    "Stomatoligist",
    "Stomotologist",
    "Stomotoligist",
    "Stomatlogist",
    "STOMATOLOGIST",
    "stomatologist",
    "StOmAtOlOgIsT",
    "Estomatólogo",
    "Stomatólogo Médico",
    # Other Possible Variations (Doctor Forms, Specialist Forms)
}

# # Define patterns (these should NOT be changed)
# stomatologist_exclusions = r'\b(?:Plastic)|(?:Physician)\b'

stomatologist_exclusions = {
    'Resident', 'resident', 'student', 'Trainee', 'Resident Doctor', 'Resident ooPhysician',
    'Intern', 'intern', 'Medical Intern', 'Fellow', 'fellow', 'Clinical Fellow', 'Medical Student',
    'Clinical Trainee', 'Trainee Doctor', 'Trainee Physician', 'Junior Doctor', 'Postgraduate Trainee',
    'Aesthetic Fellow', 'Aesthetic Trainee', 'Aesthetic Medicine Fellow', 'Aesthetic Resident'
}

# Create a mask for Stomatologist
mask_stomatologist = df['speciality'].str.contains('|'.join(stomatologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(stomatologist_exact_matches)

# mask_stomatologist_exclusions = df['speciality'].str.contains(stomatologist_exclusions, case=False, na=False, regex=True)
mask_stomatologist_exclusions = df['speciality'].isin(stomatologist_exclusions)

# Final mask: Select Stomatologist
mask_stomatologist_final = mask_stomatologist & ~mask_stomatologist_exclusions

# Store the original values that will be replaced
original_stomatologist_values = df.loc[mask_stomatologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_stomatologist_final, 'speciality'] = 'Stomatologist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Stomatologist", 'green'))
print(df.loc[mask_stomatologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_stomatologist_values = df.loc[mask_stomatologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Stomatologist", "cyan"))
for original_stomatologist_value in original_stomatologist_values:
    print(f"✅ {original_stomatologist_value} → Stomatologist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Stomatologist:", 'red'))
print(grouped_stomatologist_values)

# Print summary
matched_count_stomatologist = mask_stomatologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Stomatologist: "
        f"{matched_count_stomatologist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Dentist related titles

dentist_variants = [
    r"(?i)\bdentist\b",
    r"(?i)\bdentis\b",
    r"(?i)\bdentis\s?specialist\b",
    r"(?i)\bdentista\b",
    r"(?i)\bdentista\s?oral\b",
    r"(?i)\bdentista\s?general\b",
    r"(?i)\bdoctor\s?dentist\b",
    r"(?i)\bDDS\b",
    r"(?i)\bDMD\b",
    r"(?i)\bOdontologist\b",
    r"(?i)\bImplantologist\b",
    r"(?i)\bDentistAesthetic Physician\b",
    r"(?i)\bGeneral Dentiste\b",
    r"(?i)\bMedico Odontoiatra\b",
    r"(?i)Odontoiatra",
    r"(?i)\bDentistry- Removable Prosthodontics\b",
    r"(?i)\bBDS\b",
    r"(?i)\bB D S\b",
    r"(?i)\bOdontologa\b",
    r"(?i)\bOdontologia\b",
    r"(?i)\bDentiste\b",
    r"(?i)\bDentistry Hof\b",
    r"(?i)\bProsthodontics\b",
    r"(?i)\bDentist & Aesthetician\b",
    r"(?i)\bOdontologist\b",
    r"(?i)\bImplantologist\b",
    r"(?i)\bDentist Aesthetic Doctor\b",
    r"(?i)\bOwner Dentist\b",
    r"(?i)\bOdontologa\b",
    r"(?i)\bDmd\b",
    r"(?i)\bDentistry Hof\b",
    r"(?i)\bDentistry\b",
    r"(?i)\bDentistry- Removable Prosthodontics\b",
    r"(?i)\bB D S\b",
    r"(?i)\bSpecialist Of Prosthodontics\b",
    r"(?i)\bOrofacial Dentist\b",
    r"(?i)\bGeneral Dentist\b",
    r"(?i)\bDentista Especialista Em Harmonizacao Orofacial\b",
    r"(?i)\bDentista\b",
    r"(?i)\bOdontologia\b",
    r"(?i)\bDentiste\b",
    r"(?i)\bBds\b",
    r"(?i)\bDoctor Of Dental Medicine\b",
    r"(?i)\bPediatric Dentist\b",
    r"(?i)\bDentist Orthodontics\b",
    r"(?i)\bDental Medicine\b",
    r"(?i)\bOdontoiatraa\b",
    r"(?i)\bGeneral Dentiste\b",
    r"(?i)\bDendis\b",
    r"(?i)\bDentist - Facial Harmonization\b",
    r"(?i)\bDentist & Esthetician\b",
    r"(?i)\bDentist Aesthetic\b",
    r"(?i)\bAesthetics Dentist\b",
    r"(?i)\bProsthodontics\b",
    r"(?i)\bDental Facial Aesthetics\b",
    r"(?i)\bDentistAesthetic Physician\b",
    r"(?i)\bDental & Skincare\b",
    r"(?i)\bDoctor Of Dental Medical\b",
    r"(?i)\bCosmetic Dentist Specializing In Veneers\b",
    r"(?i)\bDentist Dmd\b",
    r"(?i)\bMedico Odontoiatra\b",
    r"(?i)\bDentista Harmonizacao Orofacial\b",
    r"(?i)\bDental Doctor\b",
    r"(?i)\bDentist & Hp\b",
    r"(?i)\bGp Dentist\b",
    r"(?i)\bMedica Dentista\b",
    r"(?i)\bAesthetic Dentist\b",
    r"(?i)\bDoctor Dentist Spec\b",
    r"(?i)\bDentist Specializing In Orofacial Harmonization\b",
    r"(?i)\bDental Professional Aesthetic Practicioner\b",
    r"(?i)\bDental Professional\b",
    r"(?i)\bProf  Esthetic Medicine Dentist\b",
    r"(?i)\bDentist Naturopath\b",
    r"(?i)\bCosmetolog Dentist\b",
    r"(?i)\bDentist Aesthetic Medicine Doctor\b",
    r"(?i)\bDentist Facial Aesthetics\b",
    r"(?i)\bDental Therapist\b",
    r"(?i)\bMaxilofacial Dentist\b",
    r"(?i)\bStaff Dentist\b",
    r"(?i)\bDentist & Aesthetic Practiononer\b",
    r"(?i)\bMedico Dentista\b",
    r"(?i)\bDentist Facial Aesthetic & Dental Implants\b",
    r"(?i)\bDental Prosthodontics\b",
    r"(?i)\bCosmetic Dentist\b",
    r"(?i)\bAesthetician & Pediatric Dentist\b",
    r"(?i)\bDental & Aestetic Doctor\b",
    r"(?i)\bDentist & Med Spa\b",
]

# Exact matches that should be updated
dentist_exact_matches = {
    "Dr. Dentist",
    "Dr. [General Dentist]",
    "Dentistry",
    'Dental Clinic',
    'Dental Professional',
    'Dental',
    'Doctor Teeth',
    'DentistFacial Aesthetics',
    'Medicina Dentaria',
    'DentistOwner',
    'GDP',
}

# Define patterns (these should NOT be changed)
dentist_exclusions = [
    r"(?i)\bMaxillo\s?Facial\b",
    r"(?i)\bMaxillo-Facial\b",
    r"(?i)\bRecruiter\b",
] # maxillo variants

# Create a mask for Dentist
mask_dentist = df['speciality'].str.contains('|'.join(dentist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(dentist_exact_matches)
mask_dentist_exclusions = df['speciality'].isin(dentist_exclusions)

# Final mask: Select Dentist
mask_dentist_final = mask_dentist & ~mask_dentist_exclusions

# Store the original values that will be replaced
original_dentist_values = df.loc[mask_dentist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_dentist_final, 'speciality'] = 'Dentist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Dentist", 'green'))
print(df.loc[mask_dentist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_dentist_values = df.loc[mask_dentist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Dentist", "cyan"))
for original_dentist_value in original_dentist_values:
    print(f"✅ {original_dentist_value} → Dentist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Dentist:", 'red'))
print(grouped_dentist_values)

# Print summary
matched_count_dentist = mask_dentist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Dentist: "
        f"{matched_count_dentist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Biomedical Physician related titles

biomedical_physician_variants = [
    r"(?i)\bBiomedical\s?Physician\b",
    r"(?i)\bBiomedical\s?Doctor\b",
    r"(?i)\bBiomedical\s?Specialist\b",
    r"(?i)\bMedical\s?Doctor\s?Specialist\b",
    r"(?i)\bBiomedical\s?Medicine\s?Physician\b",
    r"(?i)\bBiomed\s?Physician\b",
    r"(?i)\bBiomed\s?Specialist\b",
    r"(?i)\bDoctor\s?of\s?Biomedical\s?Science\b",
    r"(?i)\bAesthetic Biomedicine\b",
    r"(?i)\bBiomedicina Estetica\b",
    r"(?i)\bMedical Doctor- Regenerative Medicine\b",
    r"(?i)\bAesthetics & Regenerative Medicine\b",
]

# Exact matches that should be updated
biomedical_physician_exact_matches = {
    'Biomedical Physician',
    'Biomedical Doctor',
    'Biomedical Specialist',
    'Biomedical physician',
    'Biomed Physician',
    'Biomed Specialist',
    'Doctor of Biomedical Science',
    'Biomedical Medicine Physician',
    'Medical Doctor Specialist',
    'Biomedical Expert',
    'Biomedica',
    'Biomedical',
    'Biomedic',
    'Biomedico',

    # Case-related errors
    'biomedical physician',
    'biomedical doctor',
    'biomedical specialist',
    'biomed physician',
    'biomed specialist',
    'BIOMEDICAL PHYSICIAN',
    'BIOMEDICAL DOCTOR',
    'BIOMEDICAL SPECIALIST',
    'BIOMED PHYSICIAN',
    'BIOMED SPECIALIST',

    # Common misspellings
    'Biomedic Physician',
    'Biomedial Physician',
    'Bimolecular Physician',
    'Biodical Physician',
    'Biomedical Physiscian',
    'Biomedic Doctor',
    'Bimolecular Doctor',
    'Biomedic Specalist',
    'Biomed Specialist',
    'BioMed Doc',
    'Biomendical Physician',

    # Spanish-related exclusions
    'Médico Biomédico',
    'Doctor Biomédico',
    'Especialista en Medicina Biomédica',
    'Especialista Biomédico',
    'Médico especialista en Biomedicina',
    'Médico en Ciencias Biomédicas',
    'Doctor en Ciencias Biomédicas',
    'Médico Biomedicina',
    'Regenerative Medicine',
    'regenerative medicine',
    'Regenerative medicine',
    'Biomedica Esteta',
    'Aesthetic Biomedical',
    'Biomedicine',
    'Regenerative Preventive & Aesthetic Medicine',
    'Aesthetics Degenerative Medicine',
    'Magister Biomedis',
    'Biomedicina',
    'International Leader in Molecular Medicine',
    'Biodoctor',
    'Biomedica esteta',
}

# Define patterns for  & Resident & Professor (these should NOT be changed)
biomedical_physician_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Biomedical Physician
mask_biomedical_physician = df['speciality'].str.contains('|'.join(biomedical_physician_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(biomedical_physician_exact_matches)
mask_biomedical_physician_exclusions = df['speciality'].isin(biomedical_physician_exclusions)

# Final mask: Select Biomedical Physician
mask_biomedical_physician_final = mask_biomedical_physician & ~mask_biomedical_physician_exclusions

# Store the original values that will be replaced
original_biomedical_physician_values = df.loc[mask_biomedical_physician_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_biomedical_physician_final, 'speciality'] = 'Biomedical Physician'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Biomedical Physician", 'green'))
print(df.loc[mask_biomedical_physician_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_biomedical_physician_values = df.loc[mask_biomedical_physician_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Biomedical Physician", "cyan"))
for original_biomedical_physician_value in original_biomedical_physician_values:
    print(f"✅ {original_biomedical_physician_value} → Biomedical Physician")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Biomedical Physician:", 'red'))
print(grouped_biomedical_physician_values)

# Print summary
matched_count_biomedical_physician = mask_biomedical_physician_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Biomedical Physician: "
        f"{matched_count_biomedical_physician}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for angiologist_phlebologist related titles
angiologist_phlebologist_variants = [
    r"(?i)\bAngiologist\b",
    r"(?i)\bPhlebologist\b",
    r"(?i)\bAngiologist Specialist\b",
    r"(?i)\bPhlebologist Specialist\b",
    r"(?i)\bVenous Disease Specialist\b",
    r"(?i)\bAngiologist Consultant\b",
    r"(?i)\bPhlebologist Consultant\b",
    r"(?i)\bAngiology - Aesthetic Medicine\b",
    r"(?i)\bAesthetic & Phlebolgy\b",
    r"(?i)\bPhlebotomist\b",
]

# Exact matches that should be updated
angiologist_phlebologist_exact_matches = {
    'Angiologist',
    'Phlebologist',
    'Angiologist Specialist',
    'Phlebologist Specialist',
    'Venous Disease Specialist',
    'Angiologist Consultant',
    'Phlebologist Consultant',
    'Phlebologist Specialist Doctor',

    # Case-related errors
    'angiologist',
    'phlebologist',
    'ANGIOLOGIST',
    'PHLEBOLOGIST',
    'AnGiOlOgIsT',
    'PhLeBoLoGiSt',

    # Common misspellings
    'Angilogist',
    'Phlebologist',
    'Angiolgist',
    'Phlebollogist',
    'Phlebologiist',
    'Phlebologistt',
    'Angiologyst',
    'Phlebolgist',
    'Angiologyst Specialist',
    'Phlebology Specialist',

    # Spanish-related exclusions
    'Angiólogo',
    'Flebólogo',
    'Especialista en Angiología',
    'Especialista en Flebología',
    'Médico Angiólogo',
    'Médico Flebólogo',
    'Especialista en Enfermedades Venosas',
    'Consultor de Angiología',
    'Consultor de Flebología',
}

# Define patterns for  & Resident & Professor (these should NOT be changed)
angiologist_phlebologist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
}

# Create a mask for Angiologist Phlebologist
mask_angiologist_phlebologist = df['speciality'].str.contains('|'.join(angiologist_phlebologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(angiologist_phlebologist_exact_matches)
mask_angiologist_phlebologist_exclusions = df['speciality'].isin(angiologist_phlebologist_exclusions)

# Final mask: Select Angiologist Phlebologist
mask_angiologist_phlebologist_final = mask_angiologist_phlebologist & ~mask_angiologist_phlebologist_exclusions

# Store the original values that will be replaced
original_angiologist_phlebologist_values = df.loc[mask_angiologist_phlebologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_angiologist_phlebologist_final, 'speciality'] = 'Angiologist Phlebologist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Angiologist Phlebologist", 'green'))
print(df.loc[mask_angiologist_phlebologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_angiologist_phlebologist_values = df.loc[mask_angiologist_phlebologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Angiologist Phlebologist", "cyan"))
for original_angiologist_phlebologist_value in original_angiologist_phlebologist_values:
    print(f"✅ {original_angiologist_phlebologist_value} → Angiologist Phlebologist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Angiologist Phlebologist:", 'red'))
print(grouped_angiologist_phlebologist_values)

# Print summary
matched_count_angiologist_phlebologist = mask_angiologist_phlebologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Angiologist Phlebologist: "
        f"{matched_count_angiologist_phlebologist}",
        'red'))


import pandas as pd
import re
import unicodedata
import sys
import os
from termcolor import colored

# Define regex patterns for Dermatologist related titles

dermatologist_variants = [
    r"(?i)\bdermatologist\b",
    r"(?i)\bdermatologists\b",
    r"(?i)\bdermatologist\s?md\b",
    r"(?i)\bdermatologist\s?doctor\b",
    r"(?i)\bdermatology\s?specialist\b",
    r"(?i)\bboard-certified\s?dermatologist\b",
    r"(?i)\bcertified\s?dermatologist\b",
    r"(?i)\bskin\s?specialist\b",
    r"(?i)\bdoctor\s?of\s?dermatology\b",
    r"(?i)\bdermatology\s?physician\b",
    r"(?i)\bdermatologiste\b",
    r"(?i)\bdermatologo\b",
    r"(?i)\bdermatologo(a)?\b",
    r"(?i)\bDermatologista\b",
    r"(?i)\bDermatologia\b",
    r"(?i)\bMedica Dermatologista\b",
    r"(?i)\bDermatologue\b",
    r"(?i)\bDermatologie\b",
    r"(?i)\bDermatologa\b",
    r"(?i)\bDermatovenereologist\b",
    r"(?i)\bDermatology & Venereology\b",
    r"(?i)\bAesthetic Dermatilogy\b",
    r"(?i)\bAesthetic Dernatology\b",
    r"(?i)\bAesthetic MedicineDermatology\b",
    r"(?i)\bMedica Dematologista\b",
    r"(?i)\bDermatogist\b",
    r"(?i)\bHead Of Dermatology\b",
    r"(?i)\bMole Screening\b",
    r"(?i)\bAesthetic & Clinical Dermatology\b",
    r"(?i)\bMsc Dermatology\b",
    r"(?i)\bPedderm\b",
    r"(?i)\bDermat\b",
    r"(?i)\bDermatologyst\b",
    r"(?i)\bDermathology\b",
    r"(?i)\bDermatologi Dan Venereologi\b",
    r"(?i)\bDermatovenerologist\b",
    r"(?i)\bDermatologyst\b",
    r"(?i)\bDermatologyst\b",
    r"(?i)\bDermatovenerology\b",
    r"(?i)\bDeematology\b",
    r"(?i)\bDermotoloji\b",
    r"(?i)\bDermatolgy\b",
    r"(?i)\bSkin Medical & Aesthetic\b",
    r"(?i)\bDermatoloji\b",
    r"(?i)\bDermstolog\b",
    r"(?i)\bImmunodermatology\b",
    r"(?i)\bDeramtology\b",
    r"(?i)\bSkin Msc\b",
    r"(?i)\bFamily PracticeDermatology\b",
    r"(?i)\bDermato Funcional\b",
    r"(?i)\bDermatologiss\b",
    r"(?i)\bSkin Specialist\b",
    r"(?i)\bDr Skin Care Professional\b",
    r"(?i)\bAesthetic DermatologistAnti-Aging\b",
]

# Exact matches that should be updated
dermatologist_exact_matches = {
    "Dermatologist",
    "Dermatologists",
    "Dermatologist MD",
    "Board-Certified Dermatologist",
    "Certified Dermatologist",
    "Doctor of Dermatology",
    "Skin Specialist",
    "Dermatologo",
    "Dermatologo(a)",
    "Dermatology",
    "Derm",
    "Derma",
    "Skin",
    "Aesthetic Dermatology",
    "Skin Clinic",
    'Filler',
    'Specialist Dermatology',
    'ConsultantDermatology',
    'Dermal Clinician',
    'Dermato',
    'Dermatolog',
    'Doctor Dermatology',
    'Md PhdDermatologist',
    'Chairman Dermatology Department',
    'Aesthetics & Dermatology',
    'Chair Of The Department Of Dermatology ',
    'Medicos Dermatologistas Credenciados A SbdSbcp',
    'Skin Aesthetic',
    'Chair Of The Department Of Dermatology',
    'Dermato Venereologist',
    'Dermatology & Aesthetic Medicine',
    'Dermatology & Medical Aesthetics',
    'Deematologist',
    'Specialist In Dermatology',
    'Dermatologogist',
    'Dermatolyst',
    'Dermatoly',
    'DermatologistaDermatologia Estetica',
    'Dermato-Venereologist',
    'Consultants Dermatology',
    'Dermatology Consultant',
    'Fabia valente',
}

# Define patterns (these should NOT be changed)
dermatologist_exclusions = {
    'Resident',
    'resident',
    'student',
    'Trainee',
    'Resident Doctor',
    'Resident Physician',
    'Intern',
    'intern',
    'Medical Intern',
    'Fellow',
    'fellow',
    'Clinical Fellow',
    'Medical Student',
    'Clinical Trainee',
    'Trainee Doctor',
    'Trainee Physician',
    'Junior Doctor',
    'Postgraduate Trainee',
    'Aesthetic Fellow',
    'Aesthetic Trainee',
    'Aesthetic Medicine Fellow',
    'Aesthetic Resident',
    'Cosmetic Dermatologist',
    'Estudiante De Dermatologia  Cosmetologa En Ucrania',
    'Resident Of Dermatology & Venereology',
    'Resident Of Dermatovenereologist',
    'Dermatovenereologist Resident',
    'Consultant Dermatologist & Hair Transplant Surgeon',
    'Professor Of Dermatology & Co-Director Immunodermatology Laboratory',
    'Dermatologist Resident',
    'Residence Dermatologist',
    'Dermatologie Residence',
    'Sales Medical Dermatology Specialist',
    'Sales Executive & Immunology Dermatology Specialist',
    'Immunology Sales -Dermatology Specialist',
    'Dermatology Physician Assistant',
    'Immunology Sales Dermatology Specialist',
    'Resident dermatologist',
}

# Create a mask for Dermatologist
mask_dermatologist = df['speciality'].str.contains('|'.join(dermatologist_variants), case = False,
                                                          na = False, regex = True) | \
                            df['speciality'].isin(dermatologist_exact_matches)
mask_dermatologist_exclusions = df['speciality'].isin(dermatologist_exclusions)

# Final mask: Select Dermatologist
mask_dermatologist_final = mask_dermatologist & ~mask_dermatologist_exclusions

# Store the original values that will be replaced
original_dermatologist_values = df.loc[mask_dermatologist_final, 'speciality'].unique()

# Update the speciality column based on exact matches
df.loc[mask_dermatologist_final, 'speciality'] = 'Dermatologist'

# Check AFTER updating
print(colored("\nUnique values AFTER updating: Dermatologist", 'green'))
print(df.loc[mask_dermatologist_final, 'speciality'].unique())

# GroupBy to count how many records were replaced by the new value
grouped_dermatologist_values = df.loc[mask_dermatologist_final, 'speciality'].value_counts()

# Print the exact replacements
print(colored("\nReplaced Values: Dermatologist", "cyan"))
for original_dermatologist_value in original_dermatologist_values:
    print(f"✅ {original_dermatologist_value} → Dermatologist")

# GroupBy after replacement (to show the new value)
print(colored("\nGroupBy counts AFTER replacing with Dermatologist:", 'red'))
print(grouped_dermatologist_values)

# Print summary
matched_count_dermatologist = mask_dermatologist_final.sum()

# Print results
print(
    colored(
        f"\nTotal values matched and changed (Stage 1) to Dermatologist: "
        f"{matched_count_dermatologist}",
        'red'))